Package: padma
Type: Package
Title: Individualized Multi-Omic Pathway Deviation Scores Using
        Multiple Factor Analysis
Version: 1.22.0
Authors@R: c(
  person(given="Andrea", 
         family="Rau", 
         email="andrea.rau@inrae.fr", 
         role=c("cre", "aut"), 
         comment = c(ORCID = "0000-0001-6469-488X")),
  person(given="Regina",
         family="Manansala",
         email="manansa2@uwm.edu",
         role="aut"),
  person(given="Florence",
         family="Jaffrézic",
         email="florence.jaffrezic@inrae.fr",
         role="ctb"),
  person(given="Denis",
         family="Laloë",
         email="denis.laloe@inrae.fr",
         role="aut"),
  person(given="Paul",
         family="Auer",
         email="pauer@uwm.edu",
         role="aut"))
Depends: R (>= 4.1.0), SummarizedExperiment, S4Vectors
Imports: FactoMineR, MultiAssayExperiment, methods, graphics, stats,
        utils
Suggests: testthat, BiocStyle, knitr, rmarkdown, KEGGREST, missMDA,
        ggplot2, ggrepel, car, cowplot, reshape2
Description: 
  Use multiple factor analysis to calculate individualized pathway-centric 
  scores of deviation with respect to the sampled population based on multi-omic 
  assays (e.g., RNA-seq, copy number alterations, methylation, etc). Graphical 
  and numerical outputs are provided to identify highly aberrant individuals 
  for a particular pathway of interest, as well as the gene and omics drivers of 
  aberrant multi-omic profiles.
License: GPL (>=3)
URL: https://github.com/andreamrau/padma
https: //github.com/andreamrau/padma/issues
biocViews: Software, StatisticalMethod, PrincipalComponent,
        GeneExpression, Pathways, RNASeq, BioCarta, MethylSeq
Encoding: UTF-8
LazyData: TRUE
RoxygenNote: 7.3.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/padma
git_branch: RELEASE_3_23
git_last_commit: 080ac3b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:03:53 UTC; biocbuild
Author: Andrea Rau [cre, aut] (ORCID: <https://orcid.org/0000-0001-6469-488X>),
  Regina Manansala [aut],
  Florence Jaffrézic [ctb],
  Denis Laloë [aut],
  Paul Auer [aut]
Maintainer: Andrea Rau <andrea.rau@inrae.fr>
Built: R 4.6.0; ; 2026-04-29 04:53:58 UTC; unix

# Generated by roxygen2: do not edit by hand

export(MFA_results)
export(factorMap)
export(imputed_genes)
export(ngenes)
export(omicsContrib)
export(padma)
export(padmaRun)
export(pathway_gene_deviation)
export(pathway_name)
export(removed_genes)
exportClasses(padmaResults)
exportMethods(MFA_results)
exportMethods(imputed_genes)
exportMethods(ngenes)
exportMethods(padma)
exportMethods(pathway_gene_deviation)
exportMethods(pathway_name)
exportMethods(removed_genes)
exportMethods(show)
exportMethods(summary)
import(MultiAssayExperiment)
import(SummarizedExperiment)
importClassesFrom(S4Vectors,DataFrame)
importFrom(FactoMineR,MFA)
importFrom(S4Vectors,DataFrame)
importFrom(graphics,abline)
importFrom(graphics,barplot)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,reshape)
importFrom(stats,var)
importFrom(stats,weighted.mean)
importFrom(utils,data)
importFrom(utils,tail)

# padma: Individualized multi-omic pathway deviation scores using multiple factor analysis

<img src="inst/logos/hex_padma_v2.png" align="right" width="200" />

[![DOI](https://zenodo.org/badge/177859198.svg)](https://zenodo.org/badge/latestdoi/177859198)

The *padma* package can be installed from Bioconductor (devel) 
using the *BiocManager* package:


```
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
# The following initializes usage of Bioc devel
BiocManager::install(version='devel')
BiocManager::install("padma")
```

Note that the package can also be installed directly from GitHub:

```
library(devtools)
devtools::install_github("andreamrau/padma")
library(padma)
```

*padma* uses multiple factor analysis to calculate individualized pathway-centric scores of deviation with respect to the sampled population based on multi-omic assays (e.g., RNA-seq, copy number alterations, methylation, etc). Graphical and numerical outputs are provided to identify highly aberrant individuals for a particular pathway of interest, as well as the gene and omics drivers of aberrant multi-omic profiles.


### Reference

Rau et al. (2020) Individualized multi-omic pathway deviation scores using multiple factor analysis. Biostatistics (accepted), doi: https://doi.org/10.1101/827022.

LUAD_subset <- padma::LUAD_subset
## Create MultiAssayExperiment object with LUAD data
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))

## Run padma
run_padma <- 
  padma(mae, gene_map = padma::mirtarbase,
        pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", verbose = FALSE)

summary(run_padma)

## padma plots
\dontrun{
factorMap(run_padma, dim_x = 1, dim_y = 2)
factorMap(run_padma, dim_x = 1, dim_y = 2,
           partial_id = "TCGA-78-7536")
omicsContrib(run_padma, max_dim = 10)
}

#-----------------------------------------------------------------------
#' Calculate individualized deviation scores from multi-omic data
#'
#' @param omics_data Object of class \code{'MultiAssayExperiment'} containing 
#' omics data from n matched individuals.  
#' @param gene_map (optional) Data frame mapping
#' arbitrary biological entities (e.g. miRNAs) to genes. Contains two columns, 
#' where the first provides the IDs of the entity and
#' the second provides the IDs of the corresponding target gene.
#' By default, the miRNA-gene interactions of type 'Functional MTI' from 
#' miRTarBase are used (see the preloaded \code{'mirtarbase'} data in the 
#' package).
#' @param base_ids (optional) Sample names to be used as reference base data.
#' By default, all samples are used.
#' @param supp_ids (optional) Sample names to be used as supplementary 
#' individuals to be
#' projected onto the analysis based on the individuals identified in 
#' \code{base_ids}. By default, takes the value \code{NULL}, but should not 
#' overlap with \code{base_ids} if provided by the user.
#' @param pathway_name Character of either a KEGG pathway identifier or MSigDB 
#' pathway names (e.g., see the pathway names in the \code{'geneset'} column of 
#' the preloaded \code{msigdb} data in the package), or a vector of gene 
#' symbols.
#' @param impute If \code{TRUE}, impute missing values separately in base and
#' supplementary data using MFA as implemented in the \emph{missMDA} package; 
#' otherwise simple mean imputation is used (default).
#' @param variance_threshold Minimal variance required across samples to retain 
#' a biological entity in the analysis
#' @param full_results If \code{TRUE} (default), include full MFA results in 
#' function output; otherwise, provide concise output to save space.
#' @param verbose If \code{TRUE}, provide verbose output.
#' @param ... Optional additional arguments
#'
#' @return 
#' An S4 object of class \code{padmaResults}, where individualized pathway 
#' deviation scores are stored as the assay data, and the corresponding 
#' \{pathway name, full MFA results, number of genes, and names of imputed 
#' or filtered genes\} are stored as slots that can be retrieved using
#' the appropriate accessor functions.
#'
#' @export
#' @import MultiAssayExperiment
#' @import SummarizedExperiment
#' @importFrom FactoMineR MFA
#' @importFrom stats var
#' @importFrom utils data
#' @example /inst/examples/padma-package.R
padmaRun <- function(omics_data, gene_map = padma::mirtarbase, base_ids = NULL, 
    supp_ids = NULL, pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", 
    impute = FALSE, variance_threshold = 1e-04, full_results = TRUE, 
    verbose = TRUE, ...) {
    
    ## Preliminaries --------------------------------------------------------
    if (!is(omics_data, "MultiAssayExperiment")) 
        stop("Expecting a MultiAssayExperiment")
    
    ## Add MSigDB data that are preloaded in the package as msigdb
    MSigDB <- padma::msigdb
    
    ## If base ids are not provided, use all samples (even if user provides
    ## supp_ids)
    if (is.null(base_ids)) {
        base_ids <- rownames(colData(omics_data))
        supp_ids <- NULL
    }
    
    check <- c()
    if (length(supp_ids)) 
        check <- which(base_ids %in% supp_ids)
    if (length(check)) 
        stop("base_ids and supp_ids should be mutually exclusive.")
    
    ## Identify pathway genes
    ## -----------------------------------------------------------------
    if (length(pathway_name) == 1) {
        ## KEGG IDs
        if (length(grep("hsa", pathway_name))) {
            if (!requireNamespace("KEGGREST")) 
stop("Please load KEGGREST to automatically search for KEGG pathway gene IDs.")
            kg <- KEGGREST::keggGet(pathway_name)[[1]]
            Pathway_Name <- paste0(pathway_name, ": ", kg$NAME)
            pathway_genes <- data.frame(gene = kg$GENE)
            if (!nrow(pathway_genes)) {
                RESULTS <- NULL
                return(RESULTS)
            } else {
                pathway_genes <- unique(unlist(lapply(
                    strsplit(pathway_genes[grep(";", 
                  pathway_genes[, 1]), 1], split = "; "), function(x) x[1])))
            }
            ## C2 built-in MSigDB pathways
        } else if ((length(grep("c2_cp_", pathway_name)))) {
            Pathway_Name <- pathway_name
            pathway_genes <- unlist(strsplit(
                MSigDB[which(MSigDB$geneset == pathway_name), 
                2], split = ", "))
        } else {
            stop("No other built-in pathways currently supported.")
        }
        ## Custom pathways
    } else {
        ## pathway_name consists of a vector of gene names
        Pathway_Name <- "Custom"
        pathway_genes <- pathway_name
    }
    
    ## Subset data -------------------------------------------------------
    expanded_pathway_genes <- pathway_genes
    if (!is.null(gene_map)) {
        gene_map_subset <- gene_map[which(gene_map[, 2] %in% pathway_genes), , 
            drop = FALSE]
        gene_map_subset[, 1] <- tolower(gene_map_subset[, 1])
        entity_subset <- gene_map_subset[, 1]
        expanded_pathway_genes <- c(pathway_genes, entity_subset)
    }
    base_data <- suppressMessages(omics_data[expanded_pathway_genes, base_ids, 
        , drop = FALSE])
    supp_data <- suppressMessages(omics_data[expanded_pathway_genes, supp_ids, 
        , drop = FALSE])
    
    ## Add gene names to other entities (and duplicate) as needed
    if (!is.null(gene_map)) {
        for (j in names(base_data)) {
            ids <- rownames(experiments(base_data)[[j]])
            if (sum(ids %in% gene_map_subset[, 1])) {
                ids_tmp <- unique(
                    gene_map_subset[which(gene_map_subset[, 1] %in% 
                  ids), , drop = FALSE])
                exp_tmp <- matrix(
                  experiments(base_data)[[j]][match(ids_tmp[,1], 
                       rownames(experiments(base_data)[[j]])), ],
                  ncol = ncol(experiments(base_data)[[j]]))
                colnames(exp_tmp) <- colnames(experiments(base_data)[[j]])
                experiments(base_data)[[j]] <- exp_tmp
                rownames(experiments(base_data)[[j]]) <- paste0(ids_tmp[, 1], 
                  "_", ids_tmp[, 2])
            }
        }
    }
    
    ## Only keep samples that are represented by all assays
    base_data <- suppressMessages(intersectColumns(base_data))
    ## OCt 3, 2022 fix: intersectColumns does not work for empty mae
    if(max(unlist(lapply(assays(supp_data), ncol))) > 0) {
      supp_data <- suppressMessages(intersectColumns(supp_data))
    }

    ## Return empty results if no genes
    if(max(unlist(lapply(experiments(base_data), nrow))) == 0) {
      message("No matching genes found.")
      RESULTS <- NULL
      return(RESULTS)
    }
    
    ## Filter / impute data for entities as needed ----------------------
    nc_base <- ncol(base_data[[1]])
    nc_supp <- ncol(supp_data[[1]])
    removed_genes <- imputed_genes <- vector("list", length(names(base_data)))
    names(removed_genes) <- names(imputed_genes) <- names(base_data)
    ## If there are only base individuals, remove genes that have 0 var or
    ## all NAs
    if (!nrow(sampleMap(supp_data))) {
        for (j in names(base_data)) {
            remove_index <- unique(c(which(apply(base_data[[j]], 1, var) < 
                                               variance_threshold), 
                which(rowSums(is.na(base_data[[j]])) == nc_base)))
            if (length(remove_index)) {
                removed_genes[[j]] <- unlist(rownames(
                    base_data[[j]])[remove_index])
                experiments(base_data)[[j]] <- 
                    experiments(base_data)[[j]][-remove_index, 
                  ]
                if (verbose) {
                  cat("The following are removed from", j, 
                      "in base due to small variance or all NAs:\n")
                  print(removed_genes[[j]])
                }
            }
        }
    } else {
        ## If base + supp individuals, remove genes with small variance / NA
        ## in both. For genes with small var in base alone, impute genes with 0
        ## variability by reshuffling the minimally variant (> 10e-5) value
        for (j in names(base_data)) {
            ## Remove elements with very small variance or all NA's in both
            remove_base <- unique(c(which(apply(base_data[[j]], 1, var, 
                                                na.rm = TRUE) < 
                variance_threshold), which(rowSums(is.na(base_data[[j]])) == 
                                               nc_base)))
            remove_supp <- unique(c(which(apply(supp_data[[j]], 1, var, 
                                                na.rm = TRUE) < 
                variance_threshold), which(rowSums(is.na(supp_data[[j]])) == 
                                               nc_supp)))
            remove_index <- intersect(remove_base, remove_supp)
            if (length(remove_index)) {
                removed_genes[[j]] <- unlist(rownames(
                    base_data[[j]])[remove_index])
                experiments(base_data)[[j]] <- 
                    experiments(base_data)[[j]][-remove_index, 
                  ]
                experiments(supp_data)[[j]] <- 
                    experiments(supp_data)[[j]][-remove_index, 
                  ]
                if (verbose) {
                  cat("The following were removed from", j, 
                      "in base and supplementary due to small variance:\n")
                  print(removed_genes[[j]])
                }
            }
            
            ## Now identify elements with very small variance or all NA's 
            ## in base alone to impute
            impute_index <- unique(c(which(apply(base_data[[j]], 1, var) < 
                                               variance_threshold), 
                which(rowSums(is.na(base_data[[j]])) == nc_base)))
            if (length(impute_index)) {
                imputed_genes[[j]] <- unlist(rownames(
                    base_data[[j]])[impute_index])
                
                if (verbose) {
                  cat("The following were imputed for", j, 
                      "in base due to small variance or all NAs:\n")
                  print(imputed_genes[[j]])
                }
                wmv <- suppressWarnings(whichminpositivevar(base_data[[j]]))
                if (!is.na(wmv)) {
                  choose_impute <- unlist(base_data[[j]][wmv, ])
                } else {
                  choose_impute <- c(1, rep(0, nc_base - 1))
                }
                for (jj in seq_len(length(impute_index))) {
                  base_data[[j]][impute_index[jj], ] <- sample(choose_impute)
                }
            }
            ## Fill in remaining all NA's in supp with 0's
            supp0_index <- which(rowSums(is.na(supp_data[[j]])) == nc_supp)
            if (length(supp0_index) & nc_supp > 0) {
                supp_data[[j]][supp0_index, ] <- 0
            }
        }
    }
    
    ## Formatting data: use wideFormat ----------------------------------------
    gene_tables_base <- wideFormat(base_data, check.names = FALSE)
    colnames(gene_tables_base) <- switch_names(colnames(gene_tables_base))
    rownames(gene_tables_base) <- gene_tables_base[, 1]
    gene_tables_base <- gene_tables_base[, -1]
    ## Reorder to group by genes
    o <- order(colnames(gene_tables_base))
    gene_tables_base <- gene_tables_base[, o]
    if (nrow(colData(supp_data))) {
        gene_tables_supp <- wideFormat(supp_data, check.names = FALSE)
        colnames(gene_tables_supp) <- switch_names(colnames(gene_tables_supp))
        rownames(gene_tables_supp) <- gene_tables_supp[, 1]
        gene_tables_supp <- gene_tables_supp[, -1]
        ## Reorder to group by genes
        o <- order(colnames(gene_tables_supp))
        gene_tables_supp <- gene_tables_supp[, o]
    } else {
        gene_tables_supp <- gene_tables_base[-seq_len(nrow(gene_tables_base)), 
            ]
    }
    
    ## Remove elements with more missing elements than half the number of base
    ## individuals in either group
    remove_index <- unique(c(which(colSums(is.na(gene_tables_base)) > 0.5 * 
                                       nrow(base_data$rnaseq))))
    if (length(remove_index)) {
        gene_tables_supp <- gene_tables_supp[, -remove_index]
        gene_tables_base <- gene_tables_base[, -remove_index]
    }
    
    ## Data imputation ------------------------------------------------------
    ## Only use missMDA for data imputation if specified by the user !!!  
    ## Impute missing data for base and supp independently with missMDA 
    ## (use type = 's' to scale data)
    lgr_tab <- table(unlist(lapply(strsplit(colnames(gene_tables_base), 
                                            split = "_", 
        fixed = TRUE), function(x) x[1])))
    orig <- unique(unlist(lapply(strsplit(colnames(gene_tables_base), 
                                          split = "_", 
        fixed = TRUE), function(x) x[1])))
    lgr_tab <- lgr_tab[match(orig, names(lgr_tab))]
    lgr <- as.numeric(lgr_tab)
    names(lgr) <- names(lgr_tab)
    if (sum(is.na(gene_tables_base)) & impute) {
        if (!requireNamespace("missMDA")) 
      stop("Please load missMDA to impute missing data using using inputeMFA.")
        gene_tables_base <- missMDA::imputeMFA(gene_tables_base, group = lgr, 
            ncp = 2, type = rep("s", 
                                length(names(gene_tables_base))))$completeObs
    }
    if (sum(is.na(gene_tables_supp)) & impute) {
        if (!requireNamespace("missMDA")) 
     stop("Please load missMDA to impute missing data using using inputeMFA.")
        gene_tables_supp <- missMDA::imputeMFA(gene_tables_supp, group = lgr, 
            ncp = 2, type = rep("s", 
                                length(names(gene_tables_supp))))$completeObs
    }
    
    gene_tables_base_s <- scale(gene_tables_base, center = TRUE, scale = TRUE)
    if (nrow(gene_tables_supp)) {
        gene_tables_supp <- as(gene_tables_supp, "data.frame")
        gene_tables_supp_s <- 
            t((t(gene_tables_supp) - 
                   attributes(gene_tables_base_s)$`scaled:center`) / 
                  attributes(gene_tables_base_s)$`scaled:scale`)
    } else {
        gene_tables_supp_s <- gene_tables_supp
    }
    
    ## Combine all observations
    if (nrow(gene_tables_supp_s)) {
        gene_tables <- rbind(gene_tables_base_s, gene_tables_supp_s)
    } else {
        gene_tables <- gene_tables_base_s
    }
    
    ## Run MFA ----------------------------------------------------------------
    ## c = pre-scaled variables
    group <- c(rep("Base", nrow(gene_tables_base_s)), 
               rep("Supp", nrow(gene_tables_supp_s)))
    if (!nrow(gene_tables_supp_s)) {
        ind.sup <- NULL
    } else {
        ind.sup <- (nrow(gene_tables_base) + 1):(nrow(gene_tables_base) + 
                                                     nrow(gene_tables_supp))
    }
    total_MFA <- MFA(gene_tables, group = as.vector(lgr), 
                     type = as.vector(rep("c", 
        length(lgr))), ind.sup = ind.sup, ncp = ncol(gene_tables), 
        graph = FALSE, 
        name.group = names(lgr))
    
    ## Calculate downstream scores---------------------------------------------
    
    ## Calculate pathway deregulation score
    ps <- pathway_scores(total_MFA, ngenes = length(lgr))
    ps_df <- data.frame(primary = names(ps), group = group, 
                        pathway_deviation = ps, 
        row.names = NULL, stringsAsFactors = FALSE)
    
    ## Calculate the gene-specific pathway deregulation scores
    f <- total_MFA$ind$coord
    fk <- total_MFA$ind$coord.partiel
    gene_names <- unique(unlist(lapply(strsplit(colnames(gene_tables), "_", 
                                                fixed = TRUE), 
        function(x) x[1])))
    fk_list <- vector("list", length(gene_names))
    names(fk_list) <- gene_names
    for (g in gene_names) {
        fk_list[[g]] <- fk[grep(paste0(".", g, "$"), rownames(fk)), ]
    }
    df_final <- matrix(NA, nrow = nrow(f), ncol = length(gene_names))
    rownames(df_final) <- rownames(f)
    colnames(df_final) <- gene_names
    for (g in gene_names) {
        ## Renormalized by distance scores
        df_final[, g] <- rowSums((f * (fk_list[[g]] - f)))/sqrt(rowSums(f^2))
    }
    gs_df <- df_final
    
    ## Omics: % contribution to each axis of the MFA
    omics_contrib_MFA <- NULL
    omics_groups <- unlist(lapply(strsplit(rownames(
        total_MFA$quanti.var$contrib), 
        split = "_", fixed = TRUE), function(x) paste0(x[-1], collapse = "_")))
    
    if (!is.na(omics_groups[1])) {
        if (length(grep("_", omics_groups))) {
            omics_groups[grep("_", omics_groups)] <- 
                unlist(lapply(strsplit(omics_groups[grep("_", 
                omics_groups)], split = "_", fixed = TRUE), function(x) x[2]))
        }
        omics_contrib_MFA <- round(rowsum(total_MFA$quanti.var$contrib, 
                                          group = omics_groups), 2)
    }
    
    if (!full_results) {
        if (!is.na(omics_groups[1])) {
            omics_contrib_MFA_summary <- data.frame(
                PC_10 = round(rowSums(omics_contrib_MFA[, 
                  seq_len(min(10, ncol(omics_contrib_MFA)))]) / 
                      min(10, ncol(omics_contrib_MFA)), 2), 
                PC_all = round(rowSums(
                    omics_contrib_MFA[, seq_len(ncol(omics_contrib_MFA))]) / 
                        ncol(omics_contrib_MFA), 2))
        } else {
            omics_contrib_MFA_summary <- data.frame(PC_10 = 1, PC_all = 1)
            row.names(omics_contrib_MFA_summary) <- "single-omic"
        }
    }
    
    ## Format results ---------------------------------------------------------
    pathway_deviation <- SummarizedExperiment(ps_df)
    if (full_results) {
        ## Base individuals: perc contributions to each PC Genes: perc 
        ## contributions to each axis of the MFA Genes: Lg coefficients to show 
        ## pairwise correlations among genes Omics: perc contribution to each 
        ## axis of the MFA Full MFA results and gene tables used
        
        MFA_results <- list(eig = total_MFA$eig, 
                            ind_contrib_MFA = round(total_MFA$ind$contrib, 
            2), gene_contrib_MFA = round(total_MFA$group$contrib, 2), 
            gene_Lg_MFA = round(total_MFA$group$Lg, 
            4), omics_contrib_MFA = omics_contrib_MFA, total_MFA = total_MFA, 
            gene_tables = gene_tables)
    } else {
        ctrb <- total_MFA$ind$contrib
        ind <- min(10, ncol(ctrb))
        grp <- total_MFA$group$contrib
        ind2 <- min(10, ncol(grp))
        ## Base individuals: 
        ##   perc contributions to each first 10 PCs and all PCs Genes:
        ## perc contributions to each first 10 PCs and all PCs Omics: 
        ##   perc contribution to each first 10 PCs and all PCs
        MFA_results <- 
            list(eig = total_MFA$eig, 
                 ind_contrib_MFA_summary = data.frame(
                     PC_10 = round(rowSums(ctrb[, seq_len(ind)])/ind, 2), 
                     PC_all = round(rowSums(
                         ctrb[, seq_len(ncol(ctrb))])/ncol(ctrb), 2)), 
                 gene_contrib_MFA_summary = data.frame(
                     PC_10 = round(rowSums(grp[,seq_len(ind2)])/ind2, 2), 
                     PC_all = round(rowSums(grp[, seq_len(ncol(grp))]) / 
                                        ncol(grp), 2)), 
                 omics_contrib_MFA_summary = omics_contrib_MFA_summary)
    }
    
    RESULTS <- padmaResults(as(pathway_deviation, 
                               "RangedSummarizedExperiment"), 
        pathway_name = Pathway_Name, 
        pathway_gene_deviation = DataFrame(gs_df), 
        ngenes = length(lgr), imputed_genes = imputed_genes, 
        removed_genes = removed_genes, 
        MFA_results = MFA_results)
    return(RESULTS)
}

#-----------------------------------------------------------------------

## DO NOT EXPORT
pathway_scores <- function(MFA_results, ngenes) {
    return(deviation = sqrt(rowSums(
        rbind(MFA_results$ind$coord, MFA_results$ind.sup$coord)^2)))/ngenes
}

## DO NOT EXPORT
whichminpositivevar <- function(xx, threshold = 1e-04) {
    xxx <- apply(xx, 1, var, na.rm = TRUE)
    return(which(xxx == min(xxx[xxx > threshold], na.rm = TRUE))[1])
}

## DO NOT EXPORT
switch_names <- function(name_vector) {
    tmp <- unlist(lapply(lapply(strsplit(name_vector, split = "_", 
                                         fixed = TRUE), 
        rev), function(x) paste0(x, collapse = "_")))
    return(tmp)
}

#' Calculate individualized deviation scores from multi-omic data
#'
#' This is the primary user interface for the \code{padma} package.
#' Generic S4 methods are implemented to calculate individualized pathway
#' deviation scores on the basis of matched, multi-omic data.
#' The supported classes for input are \code{list} and 
#' \code{MultiAssayExperiment}. The output of \code{padma} is an S4 object of 
#' class \code{padmaResults}.
#'
#' @inheritParams padmaRun
#' @param object Matched multi-omic data. May be provided as (1) a 
#' \code{MultiAssayExperiment} or (2) a named \code{list}, with each
#' element corresponding to a \code{matrix} representing an omic, with 
#' biological entities in rows. Row names should include unique biological 
#' entity IDs (e.g., gene symbols, miRNA names); columns represent 
#' individuals. If more than one biological entity is used, a 
#' \code{gene_map} data.frame providing mappings between IDs and gene
#' names should be provided if the default \code{mirtarbase} is not 
#' sufficient.
#' @param colData (optional) A \code{DataFrame} or \code{data.frame} of 
#' characteristics for all biological units, to be used in creating a 
#' \code{MultiAssayExperiment} from an  \code{object} of class \code{list}
#'
#' @return
#' An S4 object of class \code{padmaResults}, where individualized pathway 
#' deviation scores are stored as the assay data, and the corresponding 
#' \{pathway name, full MFA results, number of genes, and names of imputed 
#' or filtered genes\} are stored as slots that can be retrieved using
#' the appropriate accessor functions.
#'
#' @aliases
#' padma
#' padma-methods
#' padma,list-method
#' padma,MultiAssayExperiment-method
#'
#' @author Andrea Rau
#' @export
#' @example inst/examples/padma-package.R
#' @keywords methods
#' @rdname padma
#' @docType methods
#' @importFrom methods as is new
setMethod("padma", 
          signature = signature(object = "list"), 
          definition = function(object, colData, gene_map = padma::mirtarbase, 
                                base_ids = NULL, supp_ids = NULL, 
                                pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", 
                                impute = FALSE, variance_threshold = 1e-04, 
                                full_results = TRUE, verbose = TRUE, ...) {
    omics_data <- object
    arg.user <- list(...)
    
    ## Create MultiAssayExperiment
    if ("character" %in% unique(unlist(
      lapply(object, function(x) apply(x, 2, class))))) 
      stop("One of the data.frames in the provided list contains character data.
  Did you include clinical data in the list? 
  If so, these should instead be omitted or provided via the colData argument.")
    experiments <- object
    omics_data <- suppressMessages(
      MultiAssayExperiment::MultiAssayExperiment(experiments = experiments, 
        colData = colData)
      )
    run <- padmaRun(omics_data = omics_data, gene_map = gene_map, 
                    base_ids = base_ids, 
        supp_ids = supp_ids, pathway_name = pathway_name, impute = impute, 
        variance_threshold = variance_threshold, 
        full_results = full_results, verbose = verbose, ...)
    return(run)
})


############################################################################### 
#' @rdname padma
#' @export
setMethod("padma", 
          signature = signature(object = "MultiAssayExperiment"), 
          definition = function(object, gene_map = padma::mirtarbase, 
                                base_ids = NULL, supp_ids = NULL, 
                                pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", 
                                impute = FALSE, variance_threshold = 1e-04, 
                                full_results = TRUE, verbose = TRUE, ...) {
    omics_data <- object
    run <- padmaRun(omics_data = omics_data, gene_map = gene_map, 
                    base_ids = base_ids, supp_ids = supp_ids, 
                    pathway_name = pathway_name, impute = impute, 
                    variance_threshold = variance_threshold, 
                    full_results = full_results, verbose = verbose, ...)
    return(run)
})

###############################################################################
#' Accessors for a padmaResults object.
#'
#' @docType methods
#' @rdname padmaHelpers
#' @aliases
#' pathway_name
#' pathway_name,padmaResults-method
#' MFA_results
#' MFA_results,padmaResults-method
#' ngenes
#' ngenes,padmaResults-method
#' imputed_genes
#' imputed_genes,padmaResults-method
#' removed_genes
#' removed_genes,padmaResults-method
#' pathway_gene_deviation
#' pathway_gene_deviation-method
#' show
#' show,padmaResults-method
#' 
#' @param object a \code{padmaResults} object
#' @param ... Additional optional parameters
#' @return Output varies depending on the method. 
#' @author Andrea Rau
#' @export
#' @example inst/examples/padma-package.R
setMethod("pathway_name", signature(object = "padmaResults"), 
          function(object) object@pathway_name)

#' @rdname padmaHelpers
#' @export
setMethod("MFA_results", "padmaResults", function(object) object@MFA_results)

#' @rdname padmaHelpers
#' @export
setMethod("ngenes", "padmaResults", function(object) object@ngenes)

#' @rdname padmaHelpers
#' @export
setMethod("imputed_genes", "padmaResults", 
          function(object) object@imputed_genes)

#' @rdname padmaHelpers
#' @export
setMethod("removed_genes", "padmaResults", 
          function(object) object@removed_genes)

#' @rdname padmaHelpers
#' @export
setMethod("pathway_gene_deviation", "padmaResults", 
          function(object) object@pathway_gene_deviation)

#' @rdname padmaHelpers
#' @export
setMethod("show", 
          signature = "padmaResults", 
          definition = function(object) {
            cat("An object of class ", class(object), "\n", sep = "")
            if (nrow(assay(object))) {
              cat("\nPathway multi-omic deviation scores calculated for:\n", 
                  " ", nrow(assay(object)), "individuals\n", " ", 
                  nrow(MFA_results(object)$omics), "omics\n", " ", 
                  ncol(pathway_gene_deviation(object)), "genes\n\n")
              cat("Use assay(...) to access pathway deviation scores.\n")
    }
    invisible(NULL)
})

#' Summarize results from padma
#'
#' A function to summarize the pathway deviation results from \code{padma}, 
#' using the quantiles of the calculated multi-omic pathway deviation scores.
#'
#' @rdname summary
#' @aliases
#' summary
#' summary-methods
#' summary,padmaResults-method
#'
#' @param object An object of class \code{'padmaResults'}
#' @param ... Additional arguments
#' @author Andrea Rau
#' @seealso \code{\link{padma}}
#' @references
#' Rau, A., Manansala, R., Flister, M. J., Rui, H., Jaffrézic, F., Laloë, D., 
#' and  Auer, P. L. (2019) Individualized multi-omic pathway deviation scores 
#' using multiple factor analysis bioRxiv, https://doi.org/10.1101/827022.
#'
#' @return Summary of the \code{padmaResults} object.
#' @keywords methods
#' @example /inst/examples/padma-package.R
#' @export
setMethod("summary", signature(object = "padmaResults"), 
          function(object, ...) {
    x <- object
    if (!is(x, "padmaResults")) 
        stop(paste0(sQuote("object")), " must be of class ", 
             paste0(dQuote("padmaResults")), 
            sep = "")
    cat("*************************************************\n")
    cat("Pathway multi-omic deviation score quantiles:\n")
    print(quantile(assay(x)$pathway_deviation))
    cat("\nPathway multi-omic deviation scores calculated for:\n", " ", 
        nrow(assay(object)), 
        "individuals\n", " ", nrow(MFA_results(object)$omics), "omics\n", " ", 
        ncol(pathway_gene_deviation(object)), "genes\n")
    cat("*************************************************\n")
})

#' @rdname padma
#' @export
setGeneric("padma", function(object, ...) standardGeneric("padma"))

#' @rdname padmaHelpers
#' @export
setGeneric("pathway_name", 
           function(object, ...) standardGeneric("pathway_name"))

#' @rdname padmaHelpers
#' @export
setGeneric("pathway_gene_deviation", 
           function(object, ...) standardGeneric("pathway_gene_deviation"))

#' @rdname padmaHelpers
#' @export
setGeneric("MFA_results", 
           function(object, ...) standardGeneric("MFA_results"))

#' @rdname padmaHelpers
#' @export
setGeneric("ngenes", function(object, ...) standardGeneric("ngenes"))

#' @rdname padmaHelpers
#' @export
setGeneric("imputed_genes", 
           function(object, ...) standardGeneric("imputed_genes"))

#' @rdname padmaHelpers
#' @export
setGeneric("removed_genes", 
           function(object, ...) standardGeneric("removed_genes"))

#' MSigDB canonical pathways and corresponding gene lists
#'
#' Data.frame of 1322 pathways and corresponding gene symbols included in the
#' MSigDB canonical pathways curated gene set catalog, which
#' includes genes whose products are involved in metabolic and
#' signaling pathways reported in curated public databases. This
#' specifically corresponds to the 'C2 curated gene sets' catalog
#' from MSigDB v5.2 available at
#' http://bioinf.wehi.edu.au/software/MSigDB/ as described in the
#' limma Bioconductor package.
#'
#' @docType data
#'
#' @usage data(msigdb)
#'
#' @format An object of class \code{data.frame} with two
#' columns: \code{geneset}, which provides the 1322 MSigDB curated
#' pathway names (e.g., \code{'c2_cp_BIOCARTA_41BB_PATHWAY'}) and
#' \code{symbol}, which provides the comma-separated corresponding
#' list of gene symbols.
#'
#' @keywords datasets
#'
#' @references
#' Liberzon et al. (2011) Bioinformatics 27:12, 1739-1740.
#' \url{https://doi.org/10.1093/bioinformatics/btr260}.
#'
#'
#' @source
#' \href{http://software.broadinstitute.org/gsea/msigdb/genesets.jsp?collection=CP}{MSigDB Gene sets}
#' \url{http://bioinf.wehi.edu.au/software/MSigDB/}
#'
#' @examples
#' data(msigdb)
#' head(msigdb)
"msigdb"


#' Curated miR-target interaction predictions from miRTarBase
#'
#' Data.frame of 10,754 predicted miRNA gene targets from miRTarBase
#' (version 7.0), filtered to include only predictions with the
#' 'Functional MTI' support type.
#'
#' @docType data
#'
#' @usage data(mirtarbase)
#'
#' @format An object of class \code{data.frame} with two
#' columns: \code{miRNA}, which provides the miRNA identifier
#' (e.g., \code{'hsa-miR-20a-5p'}) and
#' \code{Target Gene}, which provides the corresponding
#' predicted gene target.
#'
#' @keywords datasets
#'
#' @references
#' Chou et al. (2018) Nucleic Acids Research 46, D296-D302.
#' \url{https://doi.org/10.1093/nar/gkx1067}.
#'
#' @source
#' \url{http://mirtarbase.mbc.nctu.edu.tw/php/index.php}
#'
#' @examples
#' data(mirtarbase)
#' head(mirtarbase)
"mirtarbase"


#' Subset of batch-corrected multi-omic TCGA data in lung adenocarcinoma
#'
#' List of multi-omic (RNA-seq, copy number alterations, methylation, and
#' miRNA-seq) and phenotypic data in 144 individuals in the TCGA-LUAD
#' data for the 13 genes in the D4-GDI signaling pathway.
#'
#' @docType data
#'
#' @usage data(LUAD_subset)
#'
#' @format A named list of five objects of class \code{data.frame}
#' containing a subset of the batch-corrected multi-omic TCGA
#' data from lung adenocarcinoma, corresponding to the 13 genes
#' in the D4 GDI signaling pathway:
#' \code{'clinical'} is of dimension 144 x 55 and contains clinical
#' variables for the 144 individuals. \code{'rnaseq'}, \code{'methyl'},
#'  \code{'cna'}, and \code{'mirna'} are of dimension 13 (genes) or 1 (miRNAs) 
#'  x 144 (samples), where the row names contain the gene symbol or miRNA name.
#'
#' @keywords datasets
#'
#' @references
#' The Cancer Genome Atlas Research Network (2014) Nature 511, 543-550.
#' \url{https://doi.org/10.1038/nature13385}.
#'
#' Rau et al. (2019) bioRxiv, \url{https://doi.org/10.1101/827022}.
#'
#'
#' @source
#' \href{https://www.cancer.gov/about-nci/organization/ccg/research/structural-genomics/tcga}{TCGA}
#'
#' @examples
#' data(LUAD_subset)
#' head(LUAD_subset)
"LUAD_subset"

#' Plot an MFA factor map for individuals or partial factor map based on 
#' padma analysis
#'
#' Produce an MFA factor map for individuals, or MFA partial factor map for
#' a given individual, for a pair of dimensions provided by the user.
#'
#' @param padma_obj Output from running the \code{padma} function
#' (with \code{'full_results = TRUE'})
#' @param partial_id Index or sample name to be plotted for a 
#' partial factor map.
#' @param dim_x Dimension number of the MFA to be plotted on the x-axis.
#' @param dim_y Dimension number of the MFA to be plotted on the y-axis.
#' @param plot_ellipse If \code{TRUE}, superimpose a normal confidence ellipsis
#' on the factor map.
#' @param ggplot If \code{TRUE}, use \code{ggplot2} for plotting
#' @param repel_labels If \code{TRUE}, use \code{ggrepel} to repel sample 
#' labels from each other
#'
#' @return Plot, or factor map of class \code{ggplot} if \code{ggplot2 = TRUE}.
#' @export
#' @example inst/examples/padma-package.R
#' @importFrom graphics abline barplot layout lines mtext par points text
factorMap <- function(padma_obj, partial_id = NULL, dim_x = 1, dim_y = 2, 
                      plot_ellipse = TRUE, ggplot = TRUE, 
                      repel_labels = ifelse(ggplot == TRUE, TRUE, FALSE)) {
    if (!is(padma_obj, "padmaResults")) 
        stop("This plot function expects an object of class padmaResults.")
    test <- padma_obj
    if (is.null(MFA_results(test)$total_MFA)) 
 stop("Plotting may only be used if padma is run with full_results = TRUE.")
    if (dim_x >= ncol(MFA_results(test)$total_MFA$ind$coord)) 
        stop("dim_x must be less than the largest dimension of the MFA.")
    if (dim_y >= ncol(MFA_results(test)$total_MFA$ind$coord)) 
        stop("dim_y must be less than the largest dimension of the MFA.")
    if (repel_labels & !ggplot) 
        stop("ggplot plotting must be used if repel_labels = TRUE.")
    
    lab_1 <- lab_2 <- ticks_1 <- ticks_2 <- NULL
    
    ## Full factor map -------------------------------------------------
    if (is.null(partial_id)) {
        axis_begin_1 <- 
            floor(min(MFA_results(test)$total_MFA$ind$coord[, dim_x]))
        axis_end_1 <- 
            ceiling(max(MFA_results(test)$total_MFA$ind$coord[, dim_x]))
        axis_begin_2 <- 
            floor(min(MFA_results(test)$total_MFA$ind$coord[, dim_y]))
        axis_end_2 <- 
            ceiling(max(MFA_results(test)$total_MFA$ind$coord[, dim_y]))
        lab_frame_1 <- subset(data.frame(lab_1 = seq(axis_begin_1, axis_end_1, 
            2), zero = 0), lab_1 != 0)
        lab_frame_2 <- subset(data.frame(lab_2 = seq(axis_begin_2, axis_end_2, 
            2), zero = 0), lab_2 != 0)
        # chart junk data
        tick_frame_1 <- subset(data.frame(ticks_1 = seq(axis_begin_1, 
                                                        axis_end_1, by = 1), 
                                          zero = 0), ticks_1 != 0)
        tick_frame_2 <- subset(data.frame(ticks_2 = seq(axis_begin_2, 
                                                        axis_end_2, by = 1), 
                                          zero = 0), ticks_2 != 0)
        tick_sz <- 0.1
        tick_frame_1$m_tick_sz <- tick_frame_1$zero - tick_sz
        tick_frame_1$p_tick_sz <- tick_frame_1$zero + tick_sz
        tick_frame_2$m_tick_sz <- tick_frame_2$zero - tick_sz
        tick_frame_2$p_tick_sz <- tick_frame_2$zero + tick_sz
        
        tmp <- data.frame(
            primary = rownames(MFA_results(test)$total_MFA$ind$coord), 
            MFA_results(test)$total_MFA$ind$coord[, c(dim_x, dim_y)], 
            stringsAsFactors = FALSE)
        colnames(tmp) <- c("primary", "Dim.1", "Dim.2")
        
        ## Remove all but most extreme sample labels
        tmp_index <- which(tmp$Dim.1 < quantile(tmp$Dim.1, prob = 0.025) | 
                               tmp$Dim.1 > quantile(tmp$Dim.1, prob = 0.975) | 
                               tmp$Dim.2 < quantile(tmp$Dim.2, prob = 0.025) | 
                               tmp$Dim.2 > quantile(tmp$Dim.2, prob = 0.975))
        tmp$primary[-tmp_index] <- ""
        
        ## Plot with ggplot
        if (ggplot) {
            if (!requireNamespace("ggplot2")) 
                stop("Please load ggplot2 to plot with ggplot.")
            fig2b <- ggplot2::ggplot(tmp, 
                                     ggplot2::aes_string(x = "Dim.1", 
                                                         y = "Dim.2")) + 
                ggplot2::geom_segment(x = 0, xend = 0, 
                                      y = lab_frame_2$lab_2[1], 
                  yend = tail(lab_frame_2$lab_2, 1) + 1, size = 0.5) + 
                ggplot2::geom_segment(y = 0, yend = 0, 
                                      x = lab_frame_1$lab_1[1], 
                  xend = tail(lab_frame_1$lab_1, 1) + 1, size = 0.5) + 
                ggplot2::geom_segment(data = tick_frame_1, 
                  ggplot2::aes_string(x = "ticks_1", xend = "ticks_1", 
                                      y = "m_tick_sz", yend = "p_tick_sz")) + 
                ggplot2::geom_segment(data = tick_frame_2, 
                  ggplot2::aes_string(x = "m_tick_sz", xend = "p_tick_sz", 
                                      y = "ticks_2", 
                  yend = "ticks_2"))
            if (plot_ellipse) {
                fig2b <- fig2b + 
                    ggplot2::stat_ellipse(color = "grey90", alpha = 0.2, 
                                          geom = "polygon", lwd = 1.25, 
                                          fill = "grey90")
            }
            fig2b <- fig2b + 
                ggplot2::geom_point(size = 2, alpha = 0.6) + 
                ggplot2::theme_minimal() + 
                ggplot2::xlab(paste("MFA Dimension", dim_x)) + 
                ggplot2::ylab(paste("MFA Dimension", dim_y))
            if (repel_labels) {
                if (!requireNamespace("ggrepel")) 
                  stop("Please load ggrepel to repel text labels.")
                fig2b <- fig2b + 
                    ggrepel::geom_text_repel(
                        ggplot2::aes_string(label = "primary"))
            } else {
                fig2b <- fig2b + 
                    ggplot2::geom_text(ggplot2::aes_string(label = "primary"))
            }
            return(fig2b)
        } else {
            if (plot_ellipse) {
                if (!requireNamespace("car")) 
                  stop("Please load car to draw ellipse in base graphics.")
                car::dataEllipse(as.matrix(tmp[, 2:3]), level = 0.95, 
                                 robust = TRUE, 
                  center.pch = 0, xlab = paste("MFA Dimension", dim_x), 
                  ylab = paste("MFA Dimension", 
                    dim_y), xlim = c(axis_begin_1, axis_end_1), 
                  ylim = c(axis_begin_2, 
                    axis_end_2))
                text(x = tmp[, 2], y = tmp[, 3], label = tmp[, 1])
                abline(h = 0, lty = 2)
                abline(v = 0, lty = 2)
            } else {
                plot(x = tmp[, 2], y = tmp[, 3], xlab = paste("MFA Dimension", 
                  dim_x), ylab = paste("MFA Dimension", dim_y), 
                  xlim = c(axis_begin_1, 
                  axis_end_1), ylim = c(axis_begin_2, axis_end_2))
                text(x = tmp[, 2], y = tmp[, 3], label = tmp[, 1])
                abline(h = 0, lty = 2)
                abline(v = 0, lty = 2)
            }
        }
    } else {
        ## Partial factor map -------------------------------------------------
        i <- partial_id
        tmp <- MFA_results(test)$total_MFA$ind$coord
        df <- data.frame(rbind(cbind(rownames(tmp), colnames(tmp)[dim_x], tmp[, 
            dim_x]), cbind(rownames(tmp), colnames(tmp)[dim_y], tmp[, dim_y])), 
            check.rows = FALSE)
        colnames(df) <- c("primary", "dimension", "coord")
        
        tmp2 <- data.frame(
            primary = substr(rownames(
                MFA_results(test)$total_MFA$ind$coord.partiel), 
            1, 12), 
            gene = unlist(lapply(strsplit(
                rownames(MFA_results(test)$total_MFA$ind$coord.partiel), 
            split = ".", fixed = TRUE), 
            function(x) x[2])), 
            MFA_results(test)$total_MFA$ind$coord.partiel)
        index <- which(colnames(tmp2) %in% 
                           c(paste0("Dim.", dim_x), paste0("Dim.", dim_y)))
        pdf_choose <- tmp2[which(tmp2$primary == i), c(1, 2, index)]
        colnames(pdf_choose) <- c("primary", "gene", "Dim.1", "Dim.2")
        df_choose <- df[which(df$primary == i & df$dimension %in% 
                                  c(paste0("Dim.", dim_x), 
                                    paste0("Dim.", dim_y))), ]
        df_choose <- data.frame(primary = df_choose$primary[1], 
                                as.numeric(df_choose$coord[1]), 
                                as.numeric(df_choose$coord[2]))
        colnames(df_choose) <- c("primary", "Dim.1", "Dim.2")
        
        if(!nrow(pdf_choose))
          stop("Sample ID not found: please check the partial_id argument.")
        pdf_choose$distance <- sqrt(rowSums(pdf_choose[, -c(1, 2)]^2))
        axis_begin_1 <- floor(min(pdf_choose[, 3]))
        axis_end_1 <- ceiling(max(pdf_choose[, 3])) + 1
        axis_begin_2 <- floor(min(pdf_choose[, 4]))
        axis_end_2 <- ceiling(max(pdf_choose[, 4])) + 1
        lab_frame_1 <- subset(data.frame(lab_1 = seq(axis_begin_1, axis_end_1, 
            2), zero = 0), lab_1 != 0)
        lab_frame_2 <- subset(data.frame(lab_2 = seq(axis_begin_2, axis_end_2, 
            2), zero = 0), lab_2 != 0)
        tick_frame_1 <- subset(data.frame(ticks_1 = seq(axis_begin_1, 
                                                        axis_end_1, 
            by = 1), zero = 0), ticks_1 != 0)
        tick_frame_2 <- subset(data.frame(ticks_2 = seq(axis_begin_2, 
                                                        axis_end_2, 
            by = 1), zero = 0), ticks_2 != 0)
        tick_sz <- 0.1
        
        tick_frame_1$m_tick_sz <- tick_frame_1$zero - tick_sz
        tick_frame_1$p_tick_sz <- tick_frame_1$zero + tick_sz
        tick_frame_2$m_tick_sz <- tick_frame_2$zero - tick_sz
        tick_frame_2$p_tick_sz <- tick_frame_2$zero + tick_sz
        
        ## Plot with ggplot
        if (ggplot) {
            if (!requireNamespace("ggplot2")) 
                stop("Please load ggplot2 to plot with ggplot.")
            fig3aa <- ggplot2::ggplot(df_choose) + 
                ggplot2::geom_point(ggplot2::aes_string("Dim.1", 
                  "Dim.2"), size = 5) + 
                ggplot2::geom_point(data = pdf_choose, 
                  ggplot2::aes_string("Dim.1", 
                "Dim.2"), alpha = 0.5) + 
                ggplot2::geom_segment(
                  data = merge(pdf_choose,  df_choose, by = "primary"), 
                  ggplot2::aes_string(x = "Dim.1.x", xend = "Dim.1.y", 
                                      y = "Dim.2.x", yend = "Dim.2.y"), 
                  alpha = 0.2, lty = 2) + 
                ggplot2::geom_segment(x = 0, xend = 0, 
                                      y = lab_frame_2$lab_2[1], 
                  yend = tail(lab_frame_2$lab_2, 1) + 1, size = 0.5) + 
                ggplot2::geom_segment(y = 0, 
                  yend = 0, x = lab_frame_1$lab_1[1], 
                  xend = tail(lab_frame_1$lab_1, 1) + 1, size = 0.5) + 
                ggplot2::geom_segment(data = tick_frame_1, 
                  ggplot2::aes_string(x = "ticks_1", xend = "ticks_1", 
                                      y = "m_tick_sz", yend = "p_tick_sz")) + 
                ggplot2::geom_segment(data = tick_frame_2, 
                  ggplot2::aes_string(x = "m_tick_sz", xend = "p_tick_sz", 
                                      y = "ticks_2",  yend = "ticks_2")) + 
                ggplot2::theme_minimal() + 
                ggplot2::xlab(paste("MFA Dimension", dim_x)) + 
                ggplot2::ylab(paste("MFA Dimension", dim_y))
            if (repel_labels) {
                if (!requireNamespace("ggrepel")) 
                  stop("Please load ggrepel to repel text labels.")
                fig3aa <- fig3aa + ggrepel::geom_text_repel(data = pdf_choose, 
                  ggplot2::aes_string("Dim.1", "Dim.2", label = "gene"))
                
            } else {
                fig3aa <- fig3aa + 
                    ggplot2::geom_text(data = pdf_choose, 
                                       ggplot2::aes_string("Dim.1", "Dim.2", 
                                                           label = "gene"))
            }
            return(fig3aa)
        } else {
            ## Plot with base
            plot(df_choose$Dim.1, df_choose$Dim.2, 
                 xlab = paste("MFA Dimension", 
                dim_x), ylab = paste("MFA Dimension", dim_y), 
                xlim = c(axis_begin_1, 
                axis_end_1), ylim = c(axis_begin_2, axis_end_2))
            abline(h = 0, lty = 1)
            abline(v = 0, lty = 1)
            for (ii in seq_len(nrow(pdf_choose))) {
                lines(rbind(pdf_choose[ii, 3:4], df_choose[, -1]), lty = 2, 
                      col = "gray60")
                points(pdf_choose[ii, 3:4], lty = 2, col = "gray60", pch = 20)
                text(pdf_choose[ii, 3:4], label = pdf_choose$gene[ii])
            }
            points(df_choose$Dim.1, df_choose$Dim.2, cex = 3, pch = 20)
        }
    }
}





#' Plot the omics contribution per MFA axis and the overall weighted 
#' contribution
#'
#' Plot barplots indicating the percent contribution of each omics to each MFA 
#' dimension, as well as the overall weighted (by eigenvalue) percent 
#' contribution to the full analysis.
#'
#' @param padma_obj Output from running the \code{padma} function
#' (with \code{'full_results = TRUE'})
#' @param max_dim Maximum dimension number of the MFA to be plotted
#' @param ggplot If \code{TRUE}, use \code{ggplot2} for plotting (and
#' \code{cowplot} for combining ggplots)
#' @return Barplots of percent variance contribution, optionally of class 
#' \code{ggplot}.
#' @export
#' @example inst/examples/padma-package.R
#' @importFrom stats na.omit quantile reshape weighted.mean
#' @importFrom utils tail
omicsContrib <- function(padma_obj, 
                         max_dim = min(10, nrow(MFA_results(padma_obj)$eig)), 
                         ggplot = TRUE) {
    
    if (!is(padma_obj, "padmaResults")) 
        stop("This plot function expects an object of class padmaResults.")
    test <- padma_obj
    if (is.null(MFA_results(test)$total_MFA)) 
  stop("Plotting may only be used if padma is run with full_results = TRUE.")
    if (max_dim > ncol(MFA_results(test)$total_MFA$ind$coord)) 
        stop("max_dim must be less than the largest dimension of the MFA.")
    
    df <- data.frame(MFA_results(test)$omics_contrib_MFA)
    df$Dim.0 <- apply(df, 1, weighted.mean, 
                      w = MFA_results(test)$eig[, "eigenvalue"] / 
                          sum(MFA_results(test)$eig[, 
        "eigenvalue"]))
    df$omics <- rownames(df)
    
    df <- reshape(df, varying = -ncol(df), timevar = "dimension", 
                  v.names = "percent", 
        direction = "long", sep = ".", 
        times = as.numeric(unlist(lapply(strsplit(colnames(df)[-ncol(df)], 
            split = ".", fixed = TRUE), function(x) x[2]))))
    df$dimension <- ifelse(df$dimension == 0, -5, df$dimension)
    
    eig_df <- data.frame(
        dimension = as.numeric(substr(row.names(
          MFA_results(test)$eig), 6, 10)), 
        percentage_of_variance = as.numeric(MFA_results(test)$eig[, 2]))
    df <- merge(df, eig_df, by = "dimension", all.x = TRUE)
    df$percentage_of_variance_10 <- df$percentage_of_variance/10
    df$percentage_of_variance_round <- round(df$percentage_of_variance, 2)
    
    if (ggplot) {
        ## Plot with ggplot
        if (!requireNamespace("ggplot2") | !requireNamespace("cowplot")) 
            stop("Please load ggplot2 and cowplot to plot with ggplot.")
        
        g1 <- ggplot2::ggplot(df[which(df$dimension <= max_dim & 
                                           df$dimension > 0), ]) + 
            ggplot2::geom_bar(ggplot2::aes_string(x = "dimension", 
                                                  y = "percent", 
              fill = "omics", alpha = "percentage_of_variance_10"), 
              stat = "identity") + 
            ggplot2::geom_text(data = unique(df[which(df$dimension <= max_dim & 
                df$dimension > 0), c("dimension", "percentage_of_variance", 
                                     "percentage_of_variance_round")]), 
              ggplot2::aes_string(x = "dimension", y = 105, 
                                  label = "percentage_of_variance_round")) + 
            ggplot2::scale_alpha(guide = "none") + ggplot2::ylim(c(0, 105)) + 
            cowplot::theme_minimal_hgrid() + 
            ggplot2::scale_x_continuous(breaks = seq_len(max_dim)) + 
            ggplot2::ylab("")
        g2 <- ggplot2::ggplot(df[which(df$dimension == -5), ]) + 
            ggplot2::geom_bar(ggplot2::aes_string(x = "dimension", 
            y = "percent", fill = "omics"), stat = "identity") + 
            ggplot2::xlab("Weighted average\nacross dimensions") + 
            ggplot2::ylim(c(0, 105)) + cowplot::theme_minimal_hgrid() + 
            ggplot2::theme(axis.ticks = ggplot2::element_blank(), 
            axis.text.x = ggplot2::element_blank(), legend.position = "none")
        fig5b <- cowplot::plot_grid(g2, g1, nrow = 1, rel_widths = c(1, 5), 
                                    labels = c("", 
            ""))
        return(fig5b)
    } else {
        ## Plot with base:
        op <- par()
        layout(matrix(c(1, 2, 2, 2), nrow = 1))
        barplot(as.matrix(df[which(df$dimension == -5), c("percent"), 
                             drop = FALSE]), 
            names.arg = "", xlab = "Weighted average\nacross dimensions", 
            ylab = "percent")
        df2 <- na.omit(df[which(df$dimension <= max_dim & df$dimension > 0), 
                          c("omics", 
            "percent", "dimension")])
        df2 <- reshape(df2, v.names = "percent", idvar = "omics", 
                       timevar = "dimension", 
            direction = "wide")
        rownames(df2) <- df2[, 1]
        df2 <- df2[, -1]
        b <- barplot(as.matrix(df2), 
                     names.arg = unlist(lapply(strsplit(colnames(df2), 
            split = ".", fixed = TRUE), function(x) x[2])), xlab = "Dimension", 
            legend.text = TRUE, args.legend = list(horiz = TRUE, bty = "n", 
                                                   x = ncol(df2), 
                y = 115))
        mtext(at = b, side = 3, round(unique(df[which(df$dimension <= max_dim & 
            df$dimension > 0), "percentage_of_variance"]), 1))
        suppressWarnings(par(op))
    }
}

LUAD_subset <- padma::LUAD_subset
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))
run <- padma(mae, pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY",
             verbose = FALSE)
run_concise <- padma(mae, pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY",
                     verbose = FALSE, full_results = FALSE)

##------------------------------------------------------------
context("Test of 'padma' plotting.... testing factorMap inputs")

test_that("Function throws error if full_results = FALSE", {
  expect_error(factorMap(run_concise))
  expect_error(factorMap(run_concise, partial_id = "TCGA-78-7536"))
})

test_that("Function throws error if not a padmaResults objet", {
  expect_error(factorMap(mae))
})

test_that("Function throws error if dim_x or dim_y not appropriate values", {
  expect_error(factorMap(run, dim_x = -10, dim_y = 2))
  expect_error(factorMap(run, dim_x = 1000, dim_y = 2000))
})

test_that("Function throws error if ggrepel = TRUE but ggplot = FALSE", {
  expect_error(factorMap(run, ggrepel = TRUE, ggplot = FALSE))
})

test_that("Function throws error if incorrect sample id provided", {
  expect_error(factorMap(run, partial_id = "fake_sample"))
})

##------------------------------------------------------------
context("Test of 'padma' plotting.... testing factorMap outputs")

test_that("Function returns ggplot if ggplot = TRUE", {
  expect_is(factorMap(run, ggplot = TRUE), "ggplot")
})

##------------------------------------------------------------
context("Test of 'padma' plotting.... testing omicsContrib inputs")


test_that("Function throws error if not a padmaResults objet", {
  expect_error(omicsContrib(mae))
})

test_that("Function throws error if full_results = FALSE", {
  expect_error(omicsContrib(run_concise))
})

test_that("Function throws error if max_dim not appropriate values", {
  expect_error(omicsContrib(run, max_dim = -1000))
  expect_error(omicsContrib(run, max_dim = 1000))
})

##------------------------------------------------------------
context("Test of 'padma' plotting.... testing omicsContrib outputs")

test_that("Function returns ggplot if ggplot = TRUE", {
  expect_is(omicsContrib(run, ggplot = TRUE), "ggplot")
})

#' @rdname padmaResults
#' @export
#' @import SummarizedExperiment
setClass("padmaResults", contains = "RangedSummarizedExperiment", 
         representation = 
             representation(pathway_name = "character", 
                            pathway_gene_deviation = "DataFrame", 
                            MFA_results = "list", 
                            ngenes = "integer", 
                            imputed_genes = "list", 
                            removed_genes = "list"))


#' padmaResults object and constructor
#'
#' \code{padmaResults} is a subclass of \code{RangedSummarizedExperiment},
#' used to store the individualized pathway deviation scores as well as some 
#' additional information useful about the pathway name (\code{pathway_name}), 
#' the gene-level contributions to each deviation score 
#' (\code{pathway_gene_deviation}), a full set of 
#' outputs related to the MFA (\code{MFA_results}, and the number
#' of genes used in the analysis as well as the names of those for which data 
#' imputation or filtering was required (\code{ngenes}, \code{imputed_genes}, 
#' and \code{removed_genes}, respectively).
#'
#' This constructor function would not typically be used by 'end users'.
#' This simple class extends the \code{RangedSummarizedExperiment} class of the
#' SummarizedExperiment package
#' to allow other packages to write methods for results
#' objects from the padma package. It is used by \code{\link{padmaRun}}
#' to wrap up the results table.
#'
#' @param SummarizedExperiment a \code{RangedSummarizedExperiment} of 
#' \code{padma} results
#' @param pathway_name The name of the pathway, if applicable
#' @param pathway_gene_deviation Per-gene contributions to each 
#' individualized pathway deviation score 
#' @param MFA_results List of all detailed results from the MFA
#' @param ngenes Number of genes used in the pathway deviation score calculation
#' @param imputed_genes Names of genes, per omic, for which data imputation was 
#' used to replace missing values
#' @param removed_genes Names of genes, per omic, which were filtered from the 
#' analysis due to low variation
#' @return a padmaResults object
#' @docType class
#' @rdname padmaResults
#' @importClassesFrom S4Vectors DataFrame
#' @importFrom S4Vectors DataFrame
padmaResults <- function(SummarizedExperiment, 
                         pathway_name = NULL, 
                         pathway_gene_deviation = NULL, 
                         MFA_results = NULL, 
                         ngenes = NULL, 
                         imputed_genes = NULL, 
                         removed_genes = NULL) {
    se <- SummarizedExperiment
    if (!is(se, "RangedSummarizedExperiment")) {
     stop("'SummarizedExperiment' must be a RangedSummarizedExperiment object")
    }
    if (is.null(pathway_name)) 
        pathway_name <- ""
    if (is.null(pathway_gene_deviation)) 
        pathway_gene_deviation <- DataFrame(matrix(0, nrow = 0, ncol = 0))
    if (is.null(MFA_results)) 
        MFA_results <- list()
    if (is.null(ngenes)) 
        ngenes <- integer()
    if (is.null(imputed_genes)) 
        imputed_genes <- list()
    if (is.null(removed_genes)) 
        removed_genes <- list()
    
    object <- new("padmaResults", se, pathway_name = pathway_name, 
                  pathway_gene_deviation = pathway_gene_deviation, 
                  MFA_results = MFA_results, ngenes = ngenes, 
                  imputed_genes = imputed_genes, removed_genes = removed_genes)
    return(object)
}

LUAD_subset <- padma::LUAD_subset
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))
run <- padma(mae, pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY",
             verbose = FALSE)
run_concise <- padma(mae, pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY",
             verbose = FALSE, full_results = FALSE)

##------------------------------------------------------------
context("Test of the function 'padma'.... testing inputs")

test_that("Function throws error if unexpected input", {
  expect_error(padma(as.matrix(do.call("cbind", LUAD_subset[-1])), 
                     pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY"))
  expect_error(padma(do.call("cbind", LUAD_subset[-1])), 
                     pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY")
})

test_that("Function throws error if phenotypes in list", {
  expect_error(padma(LUAD_subset, 
                     pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY"))
})

test_that("Function throws error if overlap in base_ids and supp_ids", {
  expect_error(padma(mae, 
                     base_ids = sampleMap(mae)$primary[1:10],
                     supp_ids = sampleMap(mae)$primary[5:15],
                     pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY"))
})

test_that("Function throws error if incorrect named pathway provided", {
  expect_error(padma(mae, 
                     pathway_name = "random_pathway"))
})

##------------------------------------------------------------
context("Test of the function 'padma'.... testing outputs")

test_that("Function returns empty results if no genes found", {
  expect_message(padma(mae, 
                     pathway_name = c("gene1", "gene2")))
  expect_null(padma(mae, pathway_name = c("gene1", "gene2")))
})

test_that("Function returns padmaResults object", {
  expect_s4_class(run, "padmaResults")
  expect_named(MFA_results(run), 
               c("eig","ind_contrib_MFA","gene_contrib_MFA",
                 "gene_Lg_MFA","omics_contrib_MFA", "total_MFA", 
                 "gene_tables"))
  expect_named(MFA_results(run_concise), 
               c("eig", "ind_contrib_MFA_summary", 
                 "gene_contrib_MFA_summary", "omics_contrib_MFA_summary"))
})

---
title: "*padma* package: Quick-start guide"
author: 
- name: Andrea Rau
  affiliation: INRAE
  email: andrea.rau@inrae.fr
package: padma
date: "`r Sys.Date()`"
output: 
  BiocStyle::html_document:
    toc_float: true
  BiocStyle::pdf_document: default
vignette: >
  %\VignetteIndexEntry{padma package:Quick-start guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```


```{r, fig.align = "center", out.width = "50%", echo=FALSE}
knitr::include_graphics("hex_padma_v2.png")
```

*padma* is a package that uses multiple factor analysis to calculate 
individualized pathway-centric scores of deviation with respect to the sampled 
population based on multi-omic assays (e.g., RNA-seq, copy number alterations, 
methylation, etc). In particular, *padma* characterizes individuals with 
aberrant multi-omic profiles for a given pathway of interest and quantifies 
this deviation with respect to the sampled population using a multi-omic 
consensus representation. Graphical and numerical outputs are provided to 
identify highly aberrant individuals for a particular pathway of interest, as 
well as the gene and omics drivers of aberrant multi-omic profiles.

The method implemented in this package is described in greater detail in the 
following pre-print:

> Rau, A., Manansala, R., Flister, M. J., Rui, H., Jaffrézic, F., Laloë, 
D.&ast;, and Auer, P. L.&ast; (2019) 
Individualized multi-omic pathway deviation scores 
using multiple factor analysis bioRxiv, https://doi.org/10.1101/827022.
    &ast;These authors contributed equally to this work.
    

Below, we provide a quick-start guide using a subset of the TCGA LUAD 
multi-omic data (included with the *padma* package) to illustrate the 
functionalities and output of the method. 

# Quick start (tl;dr)

As with any R package, after installation the *padma* package is loaded as 
follows:

```{r, message = FALSE}
library(padma)
```

Multi-omic data must be formatted as either a `MultiAssayExperiment` object 
(preferred) or as a named list of `matrix` objects of matched
multi-omic data from a set of individuals, which is internally converted into 
a `MultiAssayExperiment` object. A small example dataset in list
format is provided in `LUAD_subset` for multi-omic data (clinical, RNA-seq, 
CNA, methylation, miRNA-seq) on 13 genes from the D4-GDI signaling pathway in 
individuals with lung adenocarcinoma from TCGA. In practice, multi-omic
data do not need to be subsetted prior to running *padma*; this reduced
dataset is simply used as a minimal illustrative example.

A standard *padma* analysis for the D4-GDI signalling pathway takes as input 
these pre-formatted `MultiAssayExperiment` data (called \code{mae} below)
and the name of the desired pathway. Note that data should be
normalized, batch-corrected, and if appropriate, log-transformed prior
to using *padma*. Built-in pathway names and gene lists from the MSigDB 
curated pathway set are provided in the `msigdb` data provided with *padma*.

```{r, eval = FALSE}
run_padma <- 
  padma(mae, pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY")
```

Individualized pathway deviation scores can then be accessed via the `assay` 
accessor function, and per-gene contributions to these scores can be accessed
via the `gene_deviation_scores()` accessor. In addition, a variety of plots can 
be obtained, including 
a factor map or partial factor map of individuals, and a plot of the 
percentage contribution to each component by each considered omics.

```{r, eval = FALSE}
assay(run_padma)

factorMap(run_padma)
factorMap(run_padma, partial_id = "TCGA-78-7536")
omicsContrib(run_padma)
```

Additional numerical outputs included in the `run_padma` S4 object of class 
`padmaResults` include the per-gene pathway 
deviation scores and full results from the MFA analysis. See the
following section for a more detailed description of these outputs.


# Description of *padma*

A portion of the following text is taken from 
[Rau et al. (2019)](https://doi.org/10.1101/827022).


## Overview of Multiple Factor Analysis

Multiple factor analysis (MFA) represents an extension of principal component 
analysis for the case where multiple quantitative data tables are to be 
simultaneously analyzed. In particular, the MFA approach weights each table 
individually to ensure that tables with more features or those on a different 
scale do not dominate the analysis; all features within a given table are 
given the same weight. These weights are chosen such that the first eigenvalue 
of a PCA performed on each weighted table is equal to 1, ensuring that all 
tables play an equal role in the global multi-table analysis. According to 
the desired focus of the analysis, data can be structured either with molecular 
assays (e.g., RNA-seq, methylation, miRNA-seq, copy number alterations) as 
tables (and genes as features within omics), or with genes as tables (and 
molecular assays as features within genes). The MFA weights balance the 
contributions of each omic or of each gene, respectively. In *padma*, we 
focus on the latter strategy in order to allow different omics to contribute 
to a varying degree depending on the chosen pathway.

![Figure 1 from [Rau et al. (2019)](https://doi.org/10.1101/827022), 
illustrating the *padma* approach.](padma_illustration.png)

More precisely, consider a pathway or gene set composed of $p$ genes 
(Figure 1A), each of which is measured using up to $k$ molecular assays 
(e.g., RNA-seq, methylation, miRNA-seq, copy number alterations), contained 
in the set of gene-specific matrices $X_1,\ldots, X_p$ that have the same 
$n$ matched individuals (rows) and $j_1,\ldots, j_p$ potentially unmatched 
variables (columns) in each, where $j_g \in \lbrace 1, \ldots, k\rbrace$ for 
each gene $g = 1,\ldots, p$. Because only the observations and not the 
variables are matched across data tables, genes may be represented by 
potentially different subset of omics data (e.g., only expression data for 
one gene, and expression and methylation data for another). 

In the first step, these data tables are generally standardized (i.e., 
centered and scaled). Next, an individual PCA is performed using singular 
value decomposition for each gene table $X_g$, and its largest singular 
value $\lambda_g^1$ is calculated (Figure 1B). Then, all features in each 
gene table $X_g$ are weighted by $1/\lambda_g^1$, and a global PCA is 
performed using a singular value decomposition on the concatenated set of 
weighted standardized tables, $X^\ast = \left[ \frac{X_1}{\lambda_1^1}, 
\ldots, \frac{X_p}{\lambda_p^1}\right]$  (Figure 1C). This yields a matrix of 
components (i.e., latent variables) in the observation and variable space. 

The MFA thus provides a consensus across-gene representation of the individuals 
for a given pathway, and the global PCA performed on the weighted gene tables 
decomposes the consensus variance into orthogonal variables (i.e., principal 
components) that are ordered by the proportion of variance explained by each. 
The coordinates of each individual on these components, also referred to as 
factor scores, can be used to produce factor maps to represent individuals 
in this consensus space such that smaller distances reflect greater 
similarities among individuals. In addition, partial factor scores, which 
represent the position of individuals in the consensus for a given gene, 
can also be represented in the consensus factor map; the average of partial 
factor scores across all dimensions and genes for a given individual 
corresponds to the factor score (Figure 1D). 

## Calculation of individualized pathway deviation scores

In the consensus space obtained from the MFA, the origin represents the 
``average" pathway behavior across genes, omics, and individuals; individuals 
that are projected to increasingly distant points in the factor map represent 
those with increasingly aberrant values, with respect to this average, for 
one or more of the omics measures for one or more genes in the pathway. To 
quantify these aberrant individuals, we propose an individualized pathway 
deviation score $d_i$ based on the multidimensional Euclidean distance of the 
MFA component loadings for each individual to the origin:
\begin{equation*}
d_i^2 = \sum_{\ell = 1}^L f_{i,\ell}^2,    
\end{equation*}
where  $f_{i,\ell}$ corresponds to the MFA factor score of individual $i$ in 
component $\ell$, and $L$ corresponds to the rank of $X^\ast$. Note that this 
corresponds to the weighted Euclidean distance of the scaled multi-omic data 
(for the genes in a given pathway) of each individual to the origin. These 
individualized pathway deviation scores are thus nonnegative, where smaller 
values represent individuals for whom the average multi-omic pathway variation 
is close to the average, while larger scores represent individuals with 
increasingly aberrant multi-omic pathway variation with respect to the average. 
An individual with a large pathway deviation score is thus characterized by 
one or more genes, with one or more omic measures, that explain a large 
proportion of the global correlated information across the full pathway. 

## Calculation of individualized per-gene deviation scores

In order to quantify the role played by each gene for each individual, we 
decompose the individualized pathway deviation scores into gene-level 
contributions. Recall that the average of partial factor scores across all 
MFA dimensions corresponds to each individual’s factor score. We define the 
gene-level deviation for a given individual as follows:
\begin{equation*}
d_{i,g}=\frac{\sum_{\ell = 1}^L f_{i,\ell}\left(f_{i,\ell,g}-f_{i,\ell}\right)}
{\sum_{\ell = 1}^L f_{i,\ell}^2},
\end{equation*}
where as before $f_{i,\ell}$ corresponds to the MFA factor score of individual 
$i$ in component $\ell$, $L$ corresponds to the rank of $X^\ast$, and 
$f_{i,\ell,g}$ corresponds to the MFA partial factor score of individual $i$ 
in gene $g$ in component $\ell$. Note that by construction, the contributions 
of all pathway genes to the overall deviation score sum to 0. In particular, 
per-gene contributions can take on both negative and positive values according 
to the extent to which the gene influences the deviation of the overall 
pathway score from the origin (i.e., the global center of gravity across 
individuals); large positive values correspond to tables with a large 
influence on the overall deviation of an individual, while large negative 
values correspond to genes that tend to be most similar to the global average. 

# Description of built-in datasets

## `LUAD_subset`: D4-GDI signaling pathway in the TCGA-LUAD multi-omic data

In this vignette, we focus on the example of multi-omic (RNA-seq, methylation, 
copy number alterations, and miRNA-seq) data for the D4-GDI signaling 
pathway from individuals with lung adenocarcinoma (LUAD) from The Cancer 
Genome Atlas (TCGA) database. The multi-omic TCGA data were downloaded and 
processed as [previously described](https://doi.org/10.5281/zenodo.3524080), 
including batch correction for the plate effect. The total number of 
individuals considered here is $n=144$. See 
[Rau et al. (2019)](https://doi.org/10.1101/827022) for additional details 
about data processing and mapping of miRNAs to genes.

The D4-GDP dissociation inhibitor (GDI) signaling pathway is made up of 
13 genes; it was chosen for follow-up here as it was identified as having 
individualized pathway deviation scores that are significantly positively 
correlated with smaller progression-free intervals. RNA-seq, methylation, 
and CNA measures are available for all 13 genes, with the exception of CYCS 
and PARP1, for which no methylation probes were measured the promoter region. 
In addition, miRNA-seq data were included for one predicted target pair: 
hsa-mir-421 $\rightarrow$ CASP3.

The available multi-omic data for the D4-GDI signaling pathway in TCGA LUAD 
patients is included within the *padma* package as the `LUAD_subset` object, 
which is a named list of `data.frames` each containing the relevant data 
(clinical parameters, RNA-seq, methylation, miRNA-seq, CNA) for the $p=13$ 
genes of the D4-GDI pathway in the $n=144$ individuals. 

```{r, explore_LUAD_subset}
names(LUAD_subset)

lapply(LUAD_subset, class)

lapply(LUAD_subset, dim)
```
There are some important data formatting issues that are worth mentioning here:

-  The clinical data (`LUAD_subset$clinical`) have individuals as 
rows and clinical variables as columns; all other datasets contain assay data
with biological entities (e.g., genes, miRNAs) as rows and 
individuals as columns, with appropriate row and column names. Sample
(column) names for assay data correspond to the patient barcodes that are 
provided in the `bcr_patient_barcode` column of `LUAD_subset$clinical`. 
For `LUAD_subset$mirna`, the first two 
columns represent the miRNA ID (`miRNA_lc`) and corresponding targeted gene 
symbol (`gene`). 

- A `gene_map` data.frame can be optionally included to map biological entities
(e.g. miRNAs) to gene symbols as needed. By default, a set of 10,754 
predicted miRNA gene targets with "Functional MTI" support type from 
miRTarBase (version 7.0) are used for the `gene_map`.

- All omics data have already been appropriately pre-processed before 
subsetting to this pathway (e.g., RNA-seq and miRNA-seq data have been 
normalized and log-transformed, all datasets were batch-corrected for the
plate effect).

- Although *padma* does require that all datasets have the same number of 
individuals, and that all datasets are sorted so that individuals are in the 
same order, each dataset does not need to represent every considered gene. 
Note that in these data, a single gene (CASP3) is predicted to be targeted by 
a miRNA (hsa-mir-421).

- Finally, these toy data have been subsetted to include only values relevant 
to the D4-GDI signaling pathway solely in the interest of space. In practice, 
users do not need to subset their data prior to input in *padma* or match 
miRNAs to genes, as both of these steps are performed automatically for a 
ser-provided pathway name or set of genes.


## `msigdb`: the MSigDB canonical pathway gene sets

In our work, we considered  the  pathways  included  in  the  MSigDB canonical  
pathways curated gene set catalog, which includes genes whose products are 
involved in metabolic and signaling pathways reported in curated public 
databases. We specifically used the 1322 “C2 curated gene sets” catalog 
from 601MSigDB v5.2 available at http://bioinf.wehi.edu.au/software/MSigDB/as 
described in the 
[*limma*](https://bioconductor.org/packages/release/bioc/html/limma.html) 
Bioconductor  package. For convenience, the gene sets corresponding to each 
pathway have been included in the `msigdb` data frame:

```{r, msigdb}
head(msigdb)
```
In practice, to use *padma* a user can either directly provide a vector of 
gene symbols, or the geneset name as defined in this data frame 
(e.g., `c2_cp_BIOCARTA_41BB_PATHWAY`).

## `mirtarbase`: predicted gene targets of miRNAs

*padma* integrates multi-omic data by mapping omics measures to genes in a 
given pathway. Although this assignment of values to genes is relatively 
straightforward for RNA-seq, CNA, and methylation data, a definitive mapping 
of miRNA-to-gene relationships does not exist, as miRNAs can each potentially 
target multiple genes. Many methods and databases based on text-mining or 
bioinformatics-driven approaches exist to predict miRNA-target pairs. Here, 
we make use of the  curated  miR-target  interaction  (MTI)  predictions  in 
[miRTarBase](http://mirtarbase.mbc.nctu.edu.tw/php/index.php)  (version  7.0),  
using  only exact matches for miRNA IDs and target gene symbols and predictions 
with the “Functional MTI” support type. As a reference, we have included 
these predictions in the built-in `mirtarbase` data frame:

```{r}
head(mirtarbase)
```

In practice, unless an alternative data frame is provided for the 
`gene_map` argument of *padma*, the `mirtarbase` data frame is used to 
match miRNA IDs to gene symbols.

# Running *padma*

To run *padma* for the D4-GDI pathway, we start by formatting the `LUAD_subset`
data as a `MultiAssayExperiment` object.

```{r}
library(MultiAssayExperiment)

LUAD_subset <- padma::LUAD_subset
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)

mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, 
      colData = pheno_data))
```


Next, the following command is used:

```{r, runpadma}
D4GDI <- msigdb[grep("D4GDI", msigdb$geneset), "geneset"]
run_padma <- 
  padma(mae, pathway_name = D4GDI, verbose = FALSE)
```

Alternatively, *padma* could be run directly on the list of matrices
contained in `LUAD_subset` (ensuring that the clinical data are included
as `colData`):

```{r, runpadmalist}
clinical_data <- data.frame(LUAD_subset$clinical)
rownames(clinical_data) <- clinical_data$bcr_patient_barcode

run_padma_list <- 
  padma(omics_data,
        colData = clinical_data,
        pathway_name = D4GDI,
        verbose = FALSE)
```


Note that we have used the `misgdb` built-in database to find the full name of 
the D4-GDI pathway, `"c2_cp_BIOCARTA_D4GDI_PATHWAY"`. Alternatively, a vector 
of gene symbols for the pathway of interest could instead
be provided to the `pathway_name` argument:

```{r, runpadma2}
D4GDI_genes <- unlist(strsplit(
  msigdb[grep("D4GDI", msigdb$geneset), "symbol"], ", "))
D4GDI_genes

run_padma_again <- 
  padma(mae, pathway_name = D4GDI_genes, verbose = FALSE)
```

As noted above, it is required to provide either a recognized pathway name 
(see `msigdb$geneset`) or a vector
of gene symbols. In most cases, users would provide the full multi-omics data 
in the `omics_data` argument
of `padma`, rather than a pre-subsetted dataset as is the case for the 
built-in `LUAD_subset` data.

In the following, we detail the various numerical and graphical outputs that 
an be accessed after running `padma`; note that the results are an S4 object 
of class `"padmaResults"`, an extension of the `RangedSummarizedExperiment`
class.

## Numerical outputs

The numerical outputs of `padma` include the following (which can all be 
accessed using the matching accessor functions):

There are two primary numerical outputs from `padma`:      

- **pathway deviation scores**: 
accessed via `assay(...)`,   
a `data.frame` providing the individualized pathway 
deviation scores for each individual.  The `group` column indicates whether an 
individual was included as part of the reference population (`"Base"`) or as a 
supplementary individual projected onto the reference (`"Supp"`), as indicated 
by the user in the `base_ids` and `supp_ids` arguments of `padma`.
- **pathway gene-specific deviation scores**: 
accessed via `pathway_gene_deviation(...)`, a matrix of dimension $n$ 
(nuumber of individuals) x $p$ (number of genes in the pathway) containing the 
individualized per-gene deviation scores

There are also several outputs related to the MFA itself, and obtained via 
the [*FactorMineR*](http://factominer.free.fr/) R package implementation of 
the MFA:

- `MFA_results`: 
accessed via the `MFA_results(...)`, a named list containing
detailed MFA output. When `padma` is run with 
`full_results = TRUE`, these include eigenvalues (`eig`), the percent 
contributions
of each individual, gene, and omic to the MFA components (`ind_contrib_MFA`,
`gene_contrib_MFA`, `omics_contrib_MFA`), the formatted gene tables used
as input for the MFA (`gene_tables`), 
the pairwise Lg coefficients between genes
(`gene_Lf_MFA`), and the full *FactoMineR* MFA output
(`total_MFA`), which is an object of class `"MFA"`. 
See the *FactoMineR* 
[reference manual](https://cran.r-project.org/web/packages/FactoMineR/index.html)
for a full description of this full MFA output. 
When `padma` is run with 
`full_results = FALSE`, these include eigenvalues (`eig`), and a 
summary of the cumulative contributions
of each individual, gene, and omic to the first ten and full set of MFA 
components (`ind_contrib_MFA_summary`,
`gene_contrib_MFA_summary`, `omics_contrib_MFA_summary`). In both cases,
`eig` is a matrix of dimension $c$ (number of MFA components) x 3 providing the 
respective eigenvalues, percentage of variance explained, and cumulative 
percentage of variance explained by each MFA component.                    

Finally, there are additional outputs related to the pathway itself that
can be accessed via appropriate accessor functions:

- `pathway_name`: 
accessed via `pathway_name(...)`, the
user-provided name of pathway analyzed by `padma`. If a 
vector of gene symbols was provided rather than a built-in name, `"custom"` 
is returned.  
- `ngenes`: 
accessed via `ngenes(...)`, number of genes with data available 
in the pathway                   
- `imputed_genes`: 
accessed via `imputed_genes(...)`, names of entities for 
which values were automatically imputed by `padma`             
- `removed_genes`: 
accessed via `removed_genes(...)`, names of entities
which were automatically removed by 
`padma` due to small variance (< 10e-5) or all missing values across samples.             


## Graphical outputs

*padma* includes three primary plotting functions, all of which can
be produced using *ggplot2* (default) or base graphics (using argument
`ggplot = FALSE`). Note that when *ggplot2* is used, *ggrepel* is 
additionally used to better position sample labels.

- `factorMap`: 
plot the MFA components for each individual for a selected pair 
of components. This would typically be done for the first few components, and 
serves as a visualization of the structuring of indviduals for the pathway; 
for example, below the factor map suggests that individual TCGA-78-7536 is 
likely to have a larger overall pathway deviation score given that it is 
positioned quite far from the origin (for these two axes).

```{r, factorMAP}
factorMap(run_padma, dim_x = 1, dim_y = 2)
```

The base version of this plot may be produced as follows.

```{r, factorMAP2}
factorMap(run_padma, dim_x = 1, dim_y = 2, ggplot = FALSE)
```

- `factorMap` (partial mode): 
plot the MFA partial components for one specific 
individual for a pair of components. This function would be used to zero in 
on one or several indivduals (for example, TCGA-78-7536) to visualize which 
genes appear to be pulling the overall score away from the origin. Here we 
note the seeming large influence of CASP1 and CASP3. Note that the 
individual's overall pathway score, which is represented as the large black 
point, is the same as that from the previous plot. 

```{r, factorMappartial}
factorMap(run_padma,
           partial_id = "TCGA-78-7536",
           dim_x = 1, dim_y = 2)
```

As before, *ggplot2* plotting can be optionally turned off, if desired.

```{r, factorMappartial2}
factorMap(run_padma,
           partial_id = "TCGA-78-7536",
           dim_x = 1, dim_y = 2, ggplot = FALSE)
```

- `omicsContrib`: 
visualize the contribution of each omics to the variation of 
a subset of MFA components, and summarize this contribution as a weighted 
overall average. In this plot, transparency of the bars reflects the percentage 
variance explained for each component. Note that if *ggplot2* is used,
the *cowplot* package is used to combine graphics.

```{r, omicscontrib}
omicsContrib(run_padma, max_dim = 10)
```


Once again, *ggplot2* plotting can be turned off if desired.

```{r, omicscontrib2}
omicsContrib(run_padma, max_dim = 10, ggplot = FALSE)
```


## Additional options

### Projecting supplementary individuals onto a reference consensus

Unless otherwise specified, *padma* will make use of all individuals to 
calculate pathway deviation scores. However, in some cases users may wish to 
perform the MFA on a set of reference individuals and subsequently project a 
set a supplementary individuals onto the reference consensus (e.g., in a case 
where healthy individuals are to be used as a reference, and pathway deviation 
scores of diseased individuals are to be calculated relative to them). 

In such a case, the `base_ids` and `supp_ids` arguments of `padma` can be used 
to provide the sampel names for individuals in each group, respectively. For 
instance, if the first ten individuals are in the reference group, and the 
user wishes to project individuals 15 to 20 as supplementary, the following 
code could be used:

```{r}
run_padma_supp <- 
  padma(mae, pathway_name = D4GDI, verbose = FALSE,
        base_ids = sampleMap(mae)$primary[1:10],
        supp_ids = sampleMap(mae)$primary[15:20])
```

Note that there must not be any overlap in the indices provided for `base_ids` 
and `supp_ids`.

### Missing data imputation

When missing values are included in the `omics_data` argument, by default 
*padma* uses mean imputation to replace these values prior to calculating 
individualized pathway scores. Alternatively, these values can be imputed via 
a preliminary MFA as implemented in the 
[*missMDA*}(https://cran.r-project.org/web/packages/missMDA/index.html) 
package, which can in some cases be slightly more time consuming. To do, 
the `impute` argument must be activated:

```{r}
run_padma_impute <- 
  padma(mae, pathway_name = D4GDI, 
        impute = TRUE, verbose = FALSE)
```

### Providing miRNA target predictions

As noted above, if miRNA-seq data (or other data in which biological entities
are not genes) are included in the `omics_data` argument, 
a mapping to genes must be provided to *padma* in the `gene_map` argument. 
By default, the mapping in `mirtarbase` taken from 
[miRTarBase](http://mirtarbase.mbc.nctu.edu.tw/php/index.php) is used 
within *padma*. If another mapping is desired by the user, this should be 
provided as a 2-column data frame (entity name and corresponding gene) in the 
`gene_map` argument.

### Requesting concise results from *padma*

Because *padma* stores the full set of MFA numerical results above (including 
factor components and partial factor components for all individuals), the 
full output can become quite large if run for very large pathways and/or 
looped over a large number of pathways. By default, *padma* returns this 
full set of results, but users can request a space-saving concise output via 
the `full_results` argument:

```{r, eval = FALSE}
run_padma_concise <- 
  padma(mae, pathway_name = D4GDI, full_results = FALSE)
```

# FAQ

**1. Can *padma* be used with non-continuous data, like binary per-gene** 
**somatic mutation calls?**

At the current time, *padma* is only formatted to correctly handle 
continuous data. However, the MFA approach (as implemented in the 
[*FactoMineR*](http://factominer.free.fr/) package, which is called by *padma*) 
is technically able to handle continuous, categorical, and binary data, 
so this is merely a question of extending the existing *padma* code to handle 
more general situations.

**2. Can *padma* integrate omics data that cannot be summarized to the **
**gene-level? (e.g., SNP genotyping or metabolic data)**

As it is currently written, *padma* is intended to provide inference at the 
pathway-level, where members of each pathway correspond to genes. In theory 
the MFA could be performed using each omic as a table (rather than each gene), 
in which case no gene-level summarization would be required. *padma* is not 
currently equipped to handle this more general setting.

# To-do list and ideas for work in progress

The following items are on my to-do list for improvements:

- Incorporation of known hierarchical structure among genes in a given pathway
- Addition of interactivity for exploration of *padma* results
- Extensions for highly structured groups of individuals
- Extension of *padma* to cases where non-continuous data (e.g., binary or 
categorical) can be handled
- Add in the possibility to remove certain MFA components (e.g., those 
correlated with a batch effect)
- Add in the possibility to quantify contribution of a given grouping (e.g., 
omics) across all MFA components
- Add in a full working example using 
[*curatedTCGAData*](https://bioconductor.org/packages/curatedTCGAData/).

# Session Info

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-AllClasses.R
\docType{class}
\name{padmaResults-class}
\alias{padmaResults-class}
\alias{padmaResults}
\title{padmaResults object and constructor}
\usage{
padmaResults(
  SummarizedExperiment,
  pathway_name = NULL,
  pathway_gene_deviation = NULL,
  MFA_results = NULL,
  ngenes = NULL,
  imputed_genes = NULL,
  removed_genes = NULL
)
}
\arguments{
\item{SummarizedExperiment}{a \code{RangedSummarizedExperiment} of 
\code{padma} results}

\item{pathway_name}{The name of the pathway, if applicable}

\item{pathway_gene_deviation}{Per-gene contributions to each 
individualized pathway deviation score}

\item{MFA_results}{List of all detailed results from the MFA}

\item{ngenes}{Number of genes used in the pathway deviation score calculation}

\item{imputed_genes}{Names of genes, per omic, for which data imputation was 
used to replace missing values}

\item{removed_genes}{Names of genes, per omic, which were filtered from the 
analysis due to low variation}
}
\value{
a padmaResults object
}
\description{
\code{padmaResults} is a subclass of \code{RangedSummarizedExperiment},
used to store the individualized pathway deviation scores as well as some 
additional information useful about the pathway name (\code{pathway_name}), 
the gene-level contributions to each deviation score 
(\code{pathway_gene_deviation}), a full set of 
outputs related to the MFA (\code{MFA_results}, and the number
of genes used in the analysis as well as the names of those for which data 
imputation or filtering was required (\code{ngenes}, \code{imputed_genes}, 
and \code{removed_genes}, respectively).
}
\details{
This constructor function would not typically be used by 'end users'.
This simple class extends the \code{RangedSummarizedExperiment} class of the
SummarizedExperiment package
to allow other packages to write methods for results
objects from the padma package. It is used by \code{\link{padmaRun}}
to wrap up the results table.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa-AllGenerics.R, R/methods.R
\docType{methods}
\name{pathway_name}
\alias{pathway_name}
\alias{pathway_gene_deviation}
\alias{MFA_results}
\alias{ngenes}
\alias{imputed_genes}
\alias{removed_genes}
\alias{pathway_name,padmaResults-method}
\alias{MFA_results,padmaResults-method}
\alias{ngenes,padmaResults-method}
\alias{imputed_genes,padmaResults-method}
\alias{removed_genes,padmaResults-method}
\alias{pathway_gene_deviation-method}
\alias{show}
\alias{show,padmaResults-method}
\alias{pathway_gene_deviation,padmaResults-method}
\title{Accessors for a padmaResults object.}
\usage{
pathway_name(object, ...)

pathway_gene_deviation(object, ...)

MFA_results(object, ...)

ngenes(object, ...)

imputed_genes(object, ...)

removed_genes(object, ...)

\S4method{pathway_name}{padmaResults}(object)

\S4method{MFA_results}{padmaResults}(object)

\S4method{ngenes}{padmaResults}(object)

\S4method{imputed_genes}{padmaResults}(object)

\S4method{removed_genes}{padmaResults}(object)

\S4method{pathway_gene_deviation}{padmaResults}(object)

\S4method{show}{padmaResults}(object)
}
\arguments{
\item{object}{a \code{padmaResults} object}

\item{...}{Additional optional parameters}
}
\value{
Output varies depending on the method.
}
\description{
Accessors for a padmaResults object.
}
\examples{

LUAD_subset <- padma::LUAD_subset
## Create MultiAssayExperiment object with LUAD data
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))

## Run padma
run_padma <- 
  padma(mae, gene_map = padma::mirtarbase,
        pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", verbose = FALSE)

summary(run_padma)

## padma plots
\dontrun{
factorMap(run_padma, dim_x = 1, dim_y = 2)
factorMap(run_padma, dim_x = 1, dim_y = 2,
           partial_id = "TCGA-78-7536")
omicsContrib(run_padma, max_dim = 10)
}
}
\author{
Andrea Rau
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_functions.R
\name{summary,padmaResults-method}
\alias{summary,padmaResults-method}
\alias{summary}
\alias{summary-methods}
\title{Summarize results from padma}
\usage{
\S4method{summary}{padmaResults}(object, ...)
}
\arguments{
\item{object}{An object of class \code{'padmaResults'}}

\item{...}{Additional arguments}
}
\value{
Summary of the \code{padmaResults} object.
}
\description{
A function to summarize the pathway deviation results from \code{padma}, 
using the quantiles of the calculated multi-omic pathway deviation scores.
}
\examples{

LUAD_subset <- padma::LUAD_subset
## Create MultiAssayExperiment object with LUAD data
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))

## Run padma
run_padma <- 
  padma(mae, gene_map = padma::mirtarbase,
        pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", verbose = FALSE)

summary(run_padma)

## padma plots
\dontrun{
factorMap(run_padma, dim_x = 1, dim_y = 2)
factorMap(run_padma, dim_x = 1, dim_y = 2,
           partial_id = "TCGA-78-7536")
omicsContrib(run_padma, max_dim = 10)
}
}
\references{
Rau, A., Manansala, R., Flister, M. J., Rui, H., Jaffrézic, F., Laloë, D., 
and  Auer, P. L. (2019) Individualized multi-omic pathway deviation scores 
using multiple factor analysis bioRxiv, https://doi.org/10.1101/827022.
}
\seealso{
\code{\link{padma}}
}
\author{
Andrea Rau
}
\keyword{methods}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/padma_data.R
\docType{data}
\name{mirtarbase}
\alias{mirtarbase}
\title{Curated miR-target interaction predictions from miRTarBase}
\format{
An object of class \code{data.frame} with two
columns: \code{miRNA}, which provides the miRNA identifier
(e.g., \code{'hsa-miR-20a-5p'}) and
\code{Target Gene}, which provides the corresponding
predicted gene target.
}
\source{
\url{http://mirtarbase.mbc.nctu.edu.tw/php/index.php}
}
\usage{
data(mirtarbase)
}
\description{
Data.frame of 10,754 predicted miRNA gene targets from miRTarBase
(version 7.0), filtered to include only predictions with the
'Functional MTI' support type.
}
\examples{
data(mirtarbase)
head(mirtarbase)
}
\references{
Chou et al. (2018) Nucleic Acids Research 46, D296-D302.
\url{https://doi.org/10.1093/nar/gkx1067}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/main_functions.R
\name{padmaRun}
\alias{padmaRun}
\title{Calculate individualized deviation scores from multi-omic data}
\usage{
padmaRun(
  omics_data,
  gene_map = padma::mirtarbase,
  base_ids = NULL,
  supp_ids = NULL,
  pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY",
  impute = FALSE,
  variance_threshold = 1e-04,
  full_results = TRUE,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{omics_data}{Object of class \code{'MultiAssayExperiment'} containing 
omics data from n matched individuals.}

\item{gene_map}{(optional) Data frame mapping
arbitrary biological entities (e.g. miRNAs) to genes. Contains two columns, 
where the first provides the IDs of the entity and
the second provides the IDs of the corresponding target gene.
By default, the miRNA-gene interactions of type 'Functional MTI' from 
miRTarBase are used (see the preloaded \code{'mirtarbase'} data in the 
package).}

\item{base_ids}{(optional) Sample names to be used as reference base data.
By default, all samples are used.}

\item{supp_ids}{(optional) Sample names to be used as supplementary 
individuals to be
projected onto the analysis based on the individuals identified in 
\code{base_ids}. By default, takes the value \code{NULL}, but should not 
overlap with \code{base_ids} if provided by the user.}

\item{pathway_name}{Character of either a KEGG pathway identifier or MSigDB 
pathway names (e.g., see the pathway names in the \code{'geneset'} column of 
the preloaded \code{msigdb} data in the package), or a vector of gene 
symbols.}

\item{impute}{If \code{TRUE}, impute missing values separately in base and
supplementary data using MFA as implemented in the \emph{missMDA} package; 
otherwise simple mean imputation is used (default).}

\item{variance_threshold}{Minimal variance required across samples to retain 
a biological entity in the analysis}

\item{full_results}{If \code{TRUE} (default), include full MFA results in 
function output; otherwise, provide concise output to save space.}

\item{verbose}{If \code{TRUE}, provide verbose output.}

\item{...}{Optional additional arguments}
}
\value{
An S4 object of class \code{padmaResults}, where individualized pathway 
deviation scores are stored as the assay data, and the corresponding 
\{pathway name, full MFA results, number of genes, and names of imputed 
or filtered genes\} are stored as slots that can be retrieved using
the appropriate accessor functions.
}
\description{
Calculate individualized deviation scores from multi-omic data
}
\examples{

LUAD_subset <- padma::LUAD_subset
## Create MultiAssayExperiment object with LUAD data
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))

## Run padma
run_padma <- 
  padma(mae, gene_map = padma::mirtarbase,
        pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", verbose = FALSE)

summary(run_padma)

## padma plots
\dontrun{
factorMap(run_padma, dim_x = 1, dim_y = 2)
factorMap(run_padma, dim_x = 1, dim_y = 2,
           partial_id = "TCGA-78-7536")
omicsContrib(run_padma, max_dim = 10)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.R
\name{factorMap}
\alias{factorMap}
\title{Plot an MFA factor map for individuals or partial factor map based on 
padma analysis}
\usage{
factorMap(
  padma_obj,
  partial_id = NULL,
  dim_x = 1,
  dim_y = 2,
  plot_ellipse = TRUE,
  ggplot = TRUE,
  repel_labels = ifelse(ggplot == TRUE, TRUE, FALSE)
)
}
\arguments{
\item{padma_obj}{Output from running the \code{padma} function
(with \code{'full_results = TRUE'})}

\item{partial_id}{Index or sample name to be plotted for a 
partial factor map.}

\item{dim_x}{Dimension number of the MFA to be plotted on the x-axis.}

\item{dim_y}{Dimension number of the MFA to be plotted on the y-axis.}

\item{plot_ellipse}{If \code{TRUE}, superimpose a normal confidence ellipsis
on the factor map.}

\item{ggplot}{If \code{TRUE}, use \code{ggplot2} for plotting}

\item{repel_labels}{If \code{TRUE}, use \code{ggrepel} to repel sample 
labels from each other}
}
\value{
Plot, or factor map of class \code{ggplot} if \code{ggplot2 = TRUE}.
}
\description{
Produce an MFA factor map for individuals, or MFA partial factor map for
a given individual, for a pair of dimensions provided by the user.
}
\examples{

LUAD_subset <- padma::LUAD_subset
## Create MultiAssayExperiment object with LUAD data
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))

## Run padma
run_padma <- 
  padma(mae, gene_map = padma::mirtarbase,
        pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", verbose = FALSE)

summary(run_padma)

## padma plots
\dontrun{
factorMap(run_padma, dim_x = 1, dim_y = 2)
factorMap(run_padma, dim_x = 1, dim_y = 2,
           partial_id = "TCGA-78-7536")
omicsContrib(run_padma, max_dim = 10)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_functions.R
\name{omicsContrib}
\alias{omicsContrib}
\title{Plot the omics contribution per MFA axis and the overall weighted 
contribution}
\usage{
omicsContrib(
  padma_obj,
  max_dim = min(10, nrow(MFA_results(padma_obj)$eig)),
  ggplot = TRUE
)
}
\arguments{
\item{padma_obj}{Output from running the \code{padma} function
(with \code{'full_results = TRUE'})}

\item{max_dim}{Maximum dimension number of the MFA to be plotted}

\item{ggplot}{If \code{TRUE}, use \code{ggplot2} for plotting (and
\code{cowplot} for combining ggplots)}
}
\value{
Barplots of percent variance contribution, optionally of class 
\code{ggplot}.
}
\description{
Plot barplots indicating the percent contribution of each omics to each MFA 
dimension, as well as the overall weighted (by eigenvalue) percent 
contribution to the full analysis.
}
\examples{

LUAD_subset <- padma::LUAD_subset
## Create MultiAssayExperiment object with LUAD data
omics_data <- 
  list(rnaseq = as.matrix(LUAD_subset$rnaseq),
       methyl = as.matrix(LUAD_subset$methyl),
       mirna = as.matrix(LUAD_subset$mirna),
       cna = as.matrix(LUAD_subset$cna))
pheno_data <- 
  data.frame(LUAD_subset$clinical, 
             row.names = LUAD_subset$clinical$bcr_patient_barcode)
mae <-
  suppressMessages(
    MultiAssayExperiment::MultiAssayExperiment(
      experiments = omics_data, colData = pheno_data))

## Run padma
run_padma <- 
  padma(mae, gene_map = padma::mirtarbase,
        pathway_name = "c2_cp_BIOCARTA_D4GDI_PATHWAY", verbose = FALSE)

summary(run_padma)

## padma plots
\dontrun{
factorMap(run_padma, dim_x = 1, dim_y = 2)
factorMap(run_padma, dim_x = 1, dim_y = 2,
           partial_id = "TCGA-78-7536")
omicsContrib(run_padma, max_dim = 10)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/padma_data.R
\docType{data}
\name{LUAD_subset}
\alias{LUAD_subset}
\title{Subset of batch-corrected multi-omic TCGA data in lung adenocarcinoma}
\format{
A named list of five objects of class \code{data.frame}
containing a subset of the batch-corrected multi-omic TCGA
data from lung adenocarcinoma, corresponding to the 13 genes
in the D4 GDI signaling pathway:
\code{'clinical'} is of dimension 144 x 55 and contains clinical
variables for the 144 individuals. \code{'rnaseq'}, \code{'methyl'},
 \code{'cna'}, and \code{'mirna'} are of dimension 13 (genes) or 1 (miRNAs) 
 x 144 (samples), where the row names contain the gene symbol or miRNA name.
}
\source{
\href{https://www.cancer.gov/about-nci/organization/ccg/research/structural-genomics/tcga}{TCGA}
}
\usage{
data(LUAD_subset)
}
\description{
List of multi-omic (RNA-seq, copy number alterations, methylation, and
miRNA-seq) and phenotypic data in 144 individuals in the TCGA-LUAD
data for the 13 genes in the D4-GDI signaling pathway.
}
\examples{
data(LUAD_subset)
head(LUAD_subset)
}
\references{
The Cancer Genome Atlas Research Network (2014) Nature 511, 543-550.
\url{https://doi.org/10.1038/nature13385}.

Rau et al. (2019) bioRxiv, \url{https://doi.org/10.1101/827022}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/padma_data.R
\docType{data}
\name{msigdb}
\alias{msigdb}
\title{MSigDB canonical pathways and corresponding gene lists}
\format{
An object of class \code{data.frame} with two
columns: \code{geneset}, which provides the 1322 MSigDB curated
pathway names (e.g., \code{'c2_cp_BIOCARTA_41BB_PATHWAY'}) and
\code{symbol}, which provides the comma-separated corresponding
list of gene symbols.
}
\source{
\href{http://software.broadinstitute.org/gsea/msigdb/genesets.jsp?collection=CP}{MSigDB Gene sets}
\url{http://bioinf.wehi.edu.au/software/MSigDB/}
}
\usage{
data(msigdb)
}
\description{
Data.frame of 1322 pathways and corresponding gene symbols included in the
MSigDB canonical pathways curated gene set catalog, which
includes genes whose products are involved in metabolic and
signaling pathways reported in curated public databases. This
specifically corresponds to the 'C2 curated gene sets' catalog
from MSigDB v5.2 available at
http://bioinf.wehi.edu.au/software/MSigDB/ as described in the
limma Bioconductor package.
}
\examples{
data(msigdb)
head(msigdb)
}
\references{
Liberzon et al. (2011) Bioinformatics 27:12, 1739-1740.
\url{https://doi.org/10.1093/bioinformatics/btr260}.
}
\keyword{datasets}
